from __future__ import print_function, division
from SLOPpy.subroutines.common import *
from SLOPpy.subroutines.spectral_subroutines import *
from SLOPpy.subroutines.io_subroutines import *
from SLOPpy.subroutines.fit_subroutines import *
from SLOPpy.subroutines.shortcuts import *
from SLOPpy.subroutines.rebin_subroutines import *
from astropy.convolution import convolve, Box1DKernel

__all__ = ['compute_transmission_lightcurve_average_planetRF',
           'plot_transmission_lightcurve_average_planetRF',
           'compute_transmission_lightcurve_average_stellarRF',
           'plot_transmission_lightcurve_average_stellarRF',
           'compute_transmission_lightcurve_average_observerRF',
           'plot_transmission_lightcurve_average_observerRF',
           'compute_transmission_lightcurve_average',
           'plot_transmission_lightcurve_average'
           ]

def compute_transmission_lightcurve_average_planetRF(config_in, lines_label):
    compute_transmission_lightcurve_average(config_in, lines_label, reference='planetRF')


def plot_transmission_lightcurve_average_planetRF(config_in, night_input):
    plot_transmission_lightcurve_average(config_in, night_input, reference='planetRF')


def compute_transmission_lightcurve_average_stellarRF(config_in, lines_label):
    compute_transmission_lightcurve_average(config_in, lines_label, reference='stellarRF')


def plot_transmission_lightcurve_average_stellarRF(config_in, night_input):
    plot_transmission_lightcurve_average(config_in, night_input, reference='stellarRF')


def compute_transmission_lightcurve_average_observerRF(config_in, lines_label):
    compute_transmission_lightcurve_average(config_in, lines_label, reference='observerRF')


def plot_transmission_lightcurve_average_observerRF(config_in, night_input):
    plot_transmission_lightcurve_average(config_in, night_input, reference='observerRF')


subroutine_name = 'transmission_lightcurve_average'
pick_files = 'transmission_lightcurve'

def compute_transmission_lightcurve_average(config_in, lines_label, reference='planetRF'):


    night_dict = from_config_get_nights(config_in)
    #instrument_dict = from_config_get_instrument(config_in)
    #system_dict = from_config_get_system(config_in)
    planet_dict = from_config_get_planet(config_in)
    spectral_lines = from_config_get_spectral_lines(config_in)
    lines_dict = spectral_lines[lines_label] # from_config_get_transmission_lightcurve(config_in)

    output_list = ['user',
                    'mcmc_night_MED',
                    'mcmc_night_MAP',
                    'mcmc_global_MED',
                    'mcmc_global_MAP'
                    'user_uncorrected']

    append_list = ['', '_uncorrected', '_clv_model']

    shared_data = load_from_cpickle('shared', config_in['output'])

    """ Using the line-specific range to define the transmission spectrum region """

    shared_selection = (shared_data['coadd']['wave'] >= lines_dict['range'][0]) \
        & (shared_data['coadd']['wave'] < lines_dict['range'][1])
    binned_selection = (shared_data['binned']['wave'] >= lines_dict['range'][0]) \
        & (shared_data['binned']['wave'] < lines_dict['range'][1])

    lightcurve_average_template = {
        'subroutine': subroutine_name,
        'range': lines_dict['range'],
        'wave': shared_data['coadd']['wave'][shared_selection],
        'step': shared_data['coadd']['step'][shared_selection],
        'size': int(np.sum(shared_selection)),
        'binned_wave': shared_data['binned']['wave'][binned_selection],
        'binned_step': shared_data['binned']['step'][binned_selection],
        'binned_size': int(np.sum(binned_selection))
    }

    for output_selection in output_list:
        skip_iteration = False

        try:
            lightcurve_average = load_from_cpickle(subroutine_name+'_'+reference+ '_' + output_selection, config_in['output'], lines=lines_label)
            continue
        except (FileNotFoundError, IOError):
            print("  No average transmission lightcurve found for case:{0:s}, computing now ".format(output_selection))
            print()


        lightcurve_average = lightcurve_average_template.copy()

        # doublet sodium in the lab reference frame

        """
        C stabds for central
        """
        C_bands = {}
        for passband_key, passband_val in lines_dict['passbands'].items():
            C_bands[passband_key] = {}
            for line_key, line_val in lines_dict['lines'].items():
                C_bands[passband_key][line_key] = (np.abs(lightcurve_average['wave'] - line_val)*2. <passband_val)

        """
        S stand for side
        """
        S_bands = {}
        for band_key, band_val in lines_dict['continuum'].items():
            S_bands[band_key] = (lightcurve_average['wave'] >= band_val[0]) & (lightcurve_average['wave'] <= band_val[1])


        if 'full_transit_duration' in planet_dict:
            full_transit_duration = planet_dict['total_transit_duration'][0]
        else:
            full_transit_duration = planet_dict['transit_duration'][0]

        if 'total_transit_duration' in planet_dict:
            total_transit_duration = planet_dict['total_transit_duration'][0]
        else:
            total_transit_duration = planet_dict['transit_duration'][0]

        transit_in_bins = np.linspace(
            -total_transit_duration/2./planet_dict['period'][0],
            total_transit_duration/2./planet_dict['period'][0],
            6
        )
        transit_full_bins = np.linspace(
            -full_transit_duration/2./planet_dict['period'][0],
            full_transit_duration/2./planet_dict['period'][0],
            6
        )

        transit_in_step = np.average(transit_in_bins[1:]-transit_in_bins[:-1])
        transit_full_step = np.average(transit_full_bins[1:]-transit_full_bins[:-1])


        lightcurve_average['transit_in_flag'] = []
        lightcurve_average['transit_full_flag'] = []
        lightcurve_average['transit_out_flag'] = []
        lightcurve_average['transit_in'] = {}
        lightcurve_average['transit_full'] = {}
        lightcurve_average['transit_out'] = {}
        lightcurve_average['observations'] = {'phase': []}
        lightcurve_average['bands_list'] = []
        lightcurve_average['C_bands'] = C_bands
        lightcurve_average['S_bands'] = S_bands
        lightcurve_average['bins'] = {
                'transit_in_bins': transit_in_bins,
                'transit_in_step': transit_in_step,
                'transit_full_bins': transit_full_bins,
                'transit_full_step': transit_full_step
            }

        for band_key in C_bands:
            for name_append in append_list:
                lightcurve_average['observations']['delta_' + band_key + name_append] = []
            lightcurve_average['bands_list'].extend([band_key])

        for night in night_dict:

            try:
                lightcurve = load_from_cpickle(pick_files+'_'+reference+ '_' + output_selection, config_in['output'], night, lines_label)
            except:
                skip_iteration = True
                continue

            print("compute_transmission_lightcurve            Night: ", night)


            lightcurve_average['observations']['phase'].extend(lightcurve['arrays']['observations']['phase'].tolist())

            lightcurve_average['transit_in_flag'].extend(
                lightcurve['arrays']['observations']['transit_in_flag'].tolist())
            lightcurve_average['transit_full_flag'].extend(
                lightcurve['arrays']['observations']['transit_full_flag'].tolist())
            lightcurve_average['transit_out_flag'].extend(
                lightcurve['arrays']['observations']['transit_out_flag'].tolist())

            for band_key in lightcurve_average['bands_list']:
                for name_append in append_list:
                    lightcurve_average['observations']['delta_' + band_key + name_append].extend(
                        lightcurve['arrays']['observations']['delta_' + band_key + name_append].tolist())


        if skip_iteration: continue

        sorting_index = np.argsort(lightcurve_average['observations']['phase'])
        lightcurve_average['observations']['phase'] = np.asarray(lightcurve_average['observations']['phase'])[sorting_index]
        lightcurve_average['transit_in_flag'] = np.asarray(lightcurve_average['transit_in_flag'])[sorting_index]
        lightcurve_average['transit_full_flag'] = np.asarray(lightcurve_average['transit_full_flag'])[sorting_index]
        lightcurve_average['transit_out_flag'] = np.asarray(lightcurve_average['transit_out_flag'])[sorting_index]

        lightcurve_average['transit_in']['phase'] = \
            lightcurve_average['observations']['phase'][lightcurve_average['transit_in_flag']]
        lightcurve_average['transit_full']['phase'] = \
            lightcurve_average['observations']['phase'][lightcurve_average['transit_full_flag']]
        lightcurve_average['transit_out']['phase'] = \
            lightcurve_average['observations']['phase'][lightcurve_average['transit_out_flag']]

        for band_key in lightcurve_average['bands_list']:
            for name_append in append_list:

                lightcurve_average['observations']['delta_' + band_key + name_append] = \
                    np.asarray(lightcurve_average['observations']['delta_' + band_key + name_append])[sorting_index]
                lightcurve_average['transit_in']['delta_' + band_key + name_append] = \
                    lightcurve_average['observations']['delta_' + band_key + name_append][lightcurve_average['transit_in_flag']]
                lightcurve_average['transit_full']['delta_' + band_key + name_append] = \
                    lightcurve_average['observations']['delta_' + band_key + name_append][lightcurve_average['transit_full_flag']]
                lightcurve_average['transit_out']['delta_' + band_key + name_append] = \
                    lightcurve_average['observations']['delta_' + band_key + name_append][lightcurve_average['transit_out_flag']]


        pre_duration = transit_full_bins[0] - lightcurve_average['transit_out']['phase'][0]
        if pre_duration > 0:
            nsteps_pre = int(pre_duration / transit_full_step)
            if pre_duration % transit_full_step > 0.0:
                nsteps_pre += 1
        else:
            nsteps_pre = 0

        post_duration = lightcurve_average['transit_out']['phase'][-1] - transit_full_bins[-1]
        if post_duration > 0:
            nsteps_post = int(post_duration / transit_full_step)
            if post_duration % transit_full_step > 0.0:
                nsteps_post += 1
        else:
            nsteps_post = 0

        transit_bins = np.arange(transit_full_bins[0] - nsteps_pre * transit_full_step,
                                transit_full_bins[-1] + (nsteps_post + 1.1) * transit_full_step,
                                transit_full_step)

        lightcurve_average['binned'] = {
            'observations': {
                'phase': np.zeros(len(transit_bins)),
            },
            'transit_in': {},
            'transit_full': {},
            'transit_step': {},
            'transit_out': {},
        }
        for band_key in C_bands:
            for name_append in append_list:
                lightcurve_average['binned']['observations']['delta_' + band_key + name_append] = np.zeros([len(transit_bins), 2])

        transit_out_flag = np.zeros(len(transit_bins), dtype=bool)
        transit_in_flag = np.zeros(len(transit_bins), dtype=bool)
        transit_full_flag = np.zeros(len(transit_bins), dtype=bool)

        n_a = 0
        for nb in range(0, len(transit_bins) - 1):
            sel = (lightcurve_average['observations']['phase'] >= transit_bins[nb]) \
                & (lightcurve_average['observations']['phase'] < transit_bins[nb + 1])

            if np.sum(sel) <= 0: continue
            lightcurve_average['binned']['observations']['phase'][n_a] = np.average(
                lightcurve_average['observations']['phase'][sel])

            for band_key in C_bands:
                for name_append in append_list:

                    lightcurve_average['binned']['observations']['delta_' + band_key + name_append][n_a, 0], sum_weights = np.average(
                        lightcurve_average['observations']['delta_' + band_key + name_append][sel, 0],
                        weights=1. / lightcurve_average['observations']['delta_' + band_key + name_append][sel, 1] ** 2,
                        returned=True)

                    lightcurve_average['binned']['observations']['delta_' + band_key + name_append][n_a, 1] = np.sqrt(1. / sum_weights)


            if np.abs(lightcurve_average['binned']['observations']['phase'][n_a]) >= \
                total_transit_duration/2./planet_dict['period'][0]:
                    transit_out_flag[n_a] = True
            elif np.abs(lightcurve_average['binned']['observations']['phase'][n_a]) >= \
                full_transit_duration/2./planet_dict['period'][0]:
                    transit_in_flag[n_a] = True
            else:
                transit_full_flag[n_a] = True

            n_a += 1  # bins actually computed

        lightcurve_average['binned']['transit_in']['phase'] = lightcurve_average['binned']['observations']['phase'][transit_in_flag]
        lightcurve_average['binned']['transit_full']['phase'] = lightcurve_average['binned']['observations']['phase'][transit_full_flag]
        lightcurve_average['binned']['transit_out']['phase'] = lightcurve_average['binned']['observations']['phase'][transit_out_flag]
        lightcurve_average['binned']['observations']['phase'] = lightcurve_average['binned']['observations']['phase'][:n_a]

        for band_key in C_bands:
            for name_append in append_list:
                lightcurve_average['binned']['transit_in']['delta_' + band_key + name_append] = \
                    lightcurve_average['binned']['observations']['delta_' + band_key + name_append][transit_in_flag, :]
                lightcurve_average['binned']['transit_full']['delta_' + band_key + name_append] = \
                    lightcurve_average['binned']['observations']['delta_' + band_key + name_append][transit_full_flag, :]
                lightcurve_average['binned']['transit_out']['delta_' + band_key + name_append] = \
                    lightcurve_average['binned']['observations']['delta_' + band_key + name_append][transit_out_flag, :]
                lightcurve_average['binned']['observations']['delta_' + band_key + name_append] = \
                    lightcurve_average['binned']['observations']['delta_' + band_key + name_append][:n_a, :]


        save_to_cpickle(subroutine_name + '_'+reference+ '_' + output_selection, lightcurve_average, config_in['output'], lines=lines_label)


def plot_transmission_lightcurve_average(config_in, night_input='', reference='planetRF'):

    import matplotlib.pyplot as plt

    lightcurve_average = load_from_cpickle('transmission_lightcurve_average_'+reference, config_in['output'])

    for band_key in lightcurve_average['C_bands']:
        plt.figure(figsize=(12, 6))
        plt.title('Average transmission lightcurve\n {0:s}'.format(band_key))
        plt.errorbar(lightcurve_average['observations']['phase'],
                    lightcurve_average['observations']['delta_' + band_key][:,0],
                     yerr= lightcurve_average['observations']['delta_' + band_key][:,1] ,
                     fmt='.', c='k', alpha=0.25, label='observations')
        plt.errorbar(lightcurve_average['binned']['observations']['phase'],
                    lightcurve_average['binned']['observations']['delta_' + band_key][:, 0],
                     yerr= lightcurve_average['binned']['observations']['delta_' + band_key][:,1],
                     fmt='.', c='k', alpha=1.0, label='observations')

        plt.axvspan(-1, lightcurve_average['bins']['transit_in_bins'][0], alpha=0.25, color='green')
        plt.axvspan(lightcurve_average['bins']['transit_in_bins'][-1], 1., alpha=0.25, color='green')

        plt.axhline(0, c='C1')
        plt.xlim(lightcurve_average['observations']['phase'][0]-0.01,
                 lightcurve_average['observations']['phase'][-1]+0.01)
        plt.xlabel('$\lambda$ [$\AA$]')
        plt.ylabel('$\mathcal{R}$ - 1.')
        plt.legend()
        plt.show()




from __future__ import print_function, division
from SLOPpy.subroutines.common import *
from SLOPpy.subroutines.spectral_subroutines import *
from SLOPpy.subroutines.io_subroutines import *
from SLOPpy.subroutines.fit_subroutines import *
from SLOPpy.subroutines.shortcuts import *

__all__ = ["compute_telluric_airmass_observerRF_chunks"]


def compute_telluric_airmass_observerRF_chunks(config_in):

    compute_telluric_observerRF_chunks(config_in,
                                       n_iterations=1,
                                       use_berv=False,
                                       use_reference_airmass=False,
                                       subroutine_name='compute_telluric_airmass_observerRF_chunks')


def compute_telluric_observerRF_chunks(config_in, **kwargs):

    night_dict = from_config_get_nights(config_in)

    for night in night_dict:

        print()
        print("compute_telluric_airmass_observerRF_chunks         Night: ", night)

        try:
            telluric = load_from_cpickle('telluric', config_in['output'], night)
            continue
        except:
            print("No telluric correction file found, computing now ")
            print()

        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)

        """ Retrieving the observations"""
        calib_data = load_from_cpickle('calibration_fibA', config_in['output'], night)
        input_data = retrieve_observations(config_in['output'], night, lists['observations'])
        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)

        processed = {
            'subroutine': kwargs['subroutine_name'],
            'n_orders': 0,
            'n_pixels': 0
        }

        telluric = {
            'subroutine': kwargs['subroutine_name'],
            'reference_frame': 'observer'
        }

        # There must be a more elegant way to do this, but I'm, not aware of it
        for obs in lists['observations']:

            processed[obs] = {}

            processed[obs]['e2ds_rescaling'], processed[obs]['e2ds_rescaled'], processed[obs]['e2ds_rescaled_err'] = \
                perform_rescaling(input_data[obs]['wave'],
                                  input_data[obs]['e2ds'],
                                  input_data[obs]['e2ds_err'],
                                  observational_pams['wavelength_rescaling'])

            if processed['n_orders'] == 0:
                processed['n_orders'] = input_data[obs]['orders']
                processed['n_pixels'] = input_data[obs]['wave_size']

        """ Reference airmass for iterative correction of airmass"""
        if kwargs['use_reference_airmass']:
            airmass_temp = np.zeros(lists['n_transit_in'])
            for n_obs, obs in enumerate(lists['transit_in']):
                # This is to ensure that airmass, berv and rvc are associated to the correct spectra
                airmass_temp[n_obs] = input_data[obs]['AIRMASS']
            processed['airmass_ref'] = np.average(airmass_temp)
        else:
            processed['airmass_ref'] = 0.000

        for obs in lists['observations']:
            processed[obs]['e2ds_precorrected'] = processed[obs]['e2ds_rescaled'][:]
            processed[obs]['e2ds_precorrected_err'] = input_data[obs]['e2ds_err'] / processed[obs]['e2ds_rescaling']

            """ for plotting purpose only"""
            processed[obs]['wave'] = input_data[obs]['wave']
            processed[obs]['e2ds'] = input_data[obs]['e2ds']
            processed[obs]['e2ds_err'] = input_data[obs]['e2ds_err']

        for niter in xrange(0, kwargs['n_iterations']):
            print("NITER:            ", niter)

            for obs in lists['telluric']:
                processed[obs]['logI'] = np.log(processed[obs]['e2ds_precorrected'])
                processed[obs]['logI_err'] = processed[obs]['e2ds_precorrected_err']/processed[obs]['e2ds_precorrected']

            processed['telluric'] = {}

            abs_slope = np.ones([processed['n_orders'], processed['n_pixels']], dtype=np.double)
            line_shift = np.ones([processed['n_orders'], processed['n_pixels']], dtype=np.double)
            zero_point = np.ones([processed['n_orders'], processed['n_pixels']], dtype=np.double)
            pearson_r = np.zeros([processed['n_orders'], processed['n_pixels']], dtype=np.double)
            pearson_p = np.zeros([processed['n_orders'], processed['n_pixels']], dtype=np.double)

            airmass = np.zeros(lists['n_tellurics'], dtype=np.double)
            berv = np.zeros(lists['n_tellurics'], dtype=np.double)
            rvc = np.zeros(lists['n_tellurics'], dtype=np.double)

            for n_obs, obs in enumerate(lists['telluric']):
                # This is to ensure that airmass, berv and rvc are associated to the correct spectra
                processed['telluric'][obs] = {'n_obs': n_obs}
                airmass[n_obs] = input_data[obs]['AIRMASS']
                berv[n_obs] = input_data[obs]['BERV']
                rvc[n_obs] = input_data[obs]['RVC']

            for order in xrange(0, processed['n_orders']):
                print("                                 - order ", repr(order))

                logi_array = np.empty([lists['n_tellurics'], processed['n_pixels']], dtype=np.double)
                sigi_array = np.empty([lists['n_tellurics'], processed['n_pixels']], dtype=np.double)

                for obs in lists['telluric']:
                    n_obs = processed['telluric'][obs]['n_obs']
                    logi_array[n_obs, :] = processed[obs]['logI'][order, :]
                    sigi_array[n_obs, :] = processed[obs]['logI_err'][order, :]

                """ The user has the option to select between different approaches to
                    extract the telluric absorption spectrum
                    To-Do: move this section to a subroutine for cythonization"""

                if kwargs['use_berv']:
                    if observational_pams['linear_fit_method'] == 'linear_curve_fit':

                        abs_slope[order, :], line_shift[order, :], zero_point[order, :] = \
                            berv_linear_curve_fit_modified(airmass, berv, logi_array, sigi_array, processed['n_pixels'])

                    else:
                        abs_slope[order, :], line_shift[order, :], zero_point[order, :] = \
                            berv_linear_lstsq(airmass, berv, logi_array)

                else:
                    if observational_pams['linear_fit_method']== 'linear_curve_fit':

                        abs_slope[order, :], zero_point[order, :] = \
                            airmass_linear_curve_fit(airmass, logi_array, sigi_array, processed['n_pixels'])

                    else:

                        abs_slope[order, :], zero_point[order, :] = \
                            airmass_linear_lstsq(airmass, logi_array)

                """ Saving the outcome to dictionary """
                processed['telluric']['order_'+repr(order)] = {'logi_array': logi_array, 'sigi_array': sigi_array}

            processed['telluric']['spectrum_noairmass'] = np.exp(abs_slope)

            for obs in lists['observations']:
                """ Correction of telluric lines for the average airmass value, following Wyttenbach et al. 2015 """
                processed[obs]['e2ds_corrected'] = processed[obs]['e2ds_precorrected'] / \
                                                        np.power(processed['telluric']['spectrum_noairmass'],
                                                                 input_data[obs]['AIRMASS'] -
                                                                 processed['airmass_ref'])
                processed[obs]['e2ds_corrected_err'] = processed[obs]['e2ds_precorrected_err'] / \
                                                        np.power(processed['telluric']['spectrum_noairmass'],
                                                                 input_data[obs]['AIRMASS'] -
                                                                 processed['airmass_ref'])

        for obs in lists['observations']:
            # Correction of telluric lines

            telluric[obs] = {}

            telluric[obs]['spectrum_noairmass'] = processed['telluric']['spectrum_noairmass']

            telluric[obs]['airmass'] = input_data[obs]['AIRMASS']
            telluric[obs]['airmass_ref'] = processed['airmass_ref']
            telluric[obs]['null'] = telluric[obs]['spectrum_noairmass'] < 0.001
            telluric[obs]['spectrum_noairmass'][ telluric[obs]['null']] = 1.0

            telluric[obs]['spectrum'] = np.power(processed['telluric']['spectrum_noairmass'],
                                       input_data[obs]['AIRMASS'] - processed['airmass_ref'])

            telluric[obs]['spline_noairmass'] = np.ones([input_data[obs]['n_orders'],
                                                         input_data[obs]['n_pixels']],
                                                        dtype=np.double)

            for order in xrange(0, processed['n_orders']):
                telluric[obs]['spline_noairmass'][order, :], _, _ = \
                    compute_spline(input_data[obs]['wave'][order, :],
                                   telluric[obs]['spectrum_noairmass'][order, :],
                                   0.05)

            telluric[obs]['spline'] = np.power(telluric[obs]['spline_noairmass'],
                                       input_data[obs]['AIRMASS'] - processed['airmass_ref'])

            telluric[obs]['airmass'] = input_data[obs]['AIRMASS']
            telluric[obs]['airmass_ref'] = processed['airmass_ref']
            telluric[obs]['null'] = telluric[obs]['spectrum_noairmass'] < 0.001
            telluric[obs]['spectrum_noairmass'][ telluric[obs]['null']] = 1.0

            telluric[obs]['telluric_corrected'] = processed[obs]['e2ds_corrected']
            telluric[obs]['telluric_corrected_err'] = processed[obs]['e2ds_corrected_err']

        save_to_cpickle('telluric', telluric, config_in['output'], night)
        save_to_cpickle('telluric_processed', processed, config_in['output'], night)

        print()
        print("Night ", night, " completed")



import numpy as np
from astropy.io import fits
from SLOPpy.subroutines.rebin_subroutines import *
from SLOPpy.subroutines.constants import *
from SLOPpy.subroutines.common import *

def ESPRESSO_map_orders_AB(properties, order_selection):

    n_orders_A = properties['n_orders_A']
    map_orders_A_full = np.arange(0, n_orders_A, dtype=np.int16)
    map_orders_BtoA = np.asarray(properties['orders_BtoA'])

    map_orders_A = []
    map_orders_B = []

    #if order_selection == 'red_ccd':
    #    working_A_full = map_orders_A_full[properties['red_ccd']]
    #    working_BtoA = map_orders_BtoA[properties['red_ccd']]
    if order_selection in ['red_ccd', 'blue_ccd', 'full_ccd']:
        working_A_full = map_orders_A_full[properties[order_selection]]
        working_BtoA = map_orders_BtoA[properties[order_selection]]

    elif order_selection:
        working_A_full = map_orders_A_full[order_selection]
        working_BtoA = map_orders_BtoA[order_selection]
    else:
        working_A_full = map_orders_A_full
        working_BtoA = map_orders_BtoA

    for order_A, order_B in zip(working_A_full, working_BtoA):
        if order_B < 0: continue
        map_orders_A.extend([order_A])
        map_orders_B.extend([order_B])

    return map_orders_A, map_orders_B


def ESPRESSO_give_back_selected_orders(properties, fiber, order_selection):

    map_orders_A, map_orders_B = ESPRESSO_map_orders_AB(properties, order_selection)
    if fiber != 'A':
        return map_orders_B
    else:
        return map_orders_A

def ESPRESSO_get_calib_data(archive, file_rad, night_dict, fiber='A', order_selection=None):

    calib_dict = {}

    keywords, properties = ESPRESSO_get_instrument_keywords(night_dict)

    selected_orders = ESPRESSO_give_back_selected_orders(properties, fiber, order_selection)

    map_orders_A, map_orders_B = ESPRESSO_map_orders_AB(properties, order_selection)

    if fiber=='A':
        calib_dict['fibAB_orders_match'] = map_orders_A - np.min(selected_orders)
        """ The first selected order could not have a match in fiber B, so we need to renumber from the first order of
            the input selection, not from the first order that had a match """
    else:
        calib_dict['fibAB_orders_match'] = map_orders_B - np.min(map_orders_B)
        """ Only the orders with a match in fiber A are read in the first place, so we can safely rescale with respect
            to the number of the first order in the matched list """

    if properties['use_ESO_deblazed']:
        if properties['use_ESO_telluric_correction']:
            e2ds_fits = fits.open(archive+'/'+file_rad+'_S2D_TELL_CORR_'+fiber+'.fits')
        elif properties['use_ESO_sky_correction']:
            e2ds_fits = fits.open(archive+'/'+file_rad+'_S2D_SKYSUB_'+fiber+'.fits')
        else:
            e2ds_fits = fits.open(archive+'/'+file_rad+'_S2D_'+fiber+'.fits')

        blaze = np.ones_like(e2ds_fits[1].data[properties['orders_regroup'], :])
        e2ds_fits.close()
    else:
        e2ds_fits_deblazed = fits.open(archive+'/'+file_rad+'_S2D_'+fiber+'.fits')
        e2ds_fits= fits.open(archive+'/'+file_rad+'_S2D_BLAZE_'+fiber+'.fits')

        blaze_full = e2ds_fits[1].data / e2ds_fits_deblazed[1].data
        blaze = blaze_full[properties['orders_regroup'], :]

        e2ds_fits_deblazed.close()
        e2ds_fits.close()

    calib_dict['blaze'] = blaze[selected_orders, :]

    # getting lamp file
    #try:
    #    lamp_fits = fits.open(archive + '/' + blaze_file[:29] + '_lamp_' + fiber + '.fits')
    #    calib_dict['lamp'] = lamp_fits[0].data[selected_orders, :]
    #    lamp_fits.close()
    #except:
    #    print("lamp files not available, sky correction cannot not be performed")

    calib_dict['n_pixels'] = np.shape(blaze)[1]
    calib_dict['n_orders'] = len(selected_orders)

    return calib_dict


def ESPRESSO_get_input_data(archive, file_rad, night_dict, fiber='A', skip_ccf=None, skip_s1d=True, order_selection=None):

    input_dict = {'header':{}}
    input_s1d = {'header':{}}

    keywords, properties = ESPRESSO_get_instrument_keywords(night_dict)

    selected_orders = ESPRESSO_give_back_selected_orders(properties, fiber, order_selection)

    if properties['use_ESO_deblazed']:
        if properties['use_ESO_telluric_correction']:
            e2ds_fits = fits.open(archive+'/'+file_rad+'_S2D_TELL_CORR_'+fiber+'.fits')
        elif properties['use_ESO_sky_correction']:
            e2ds_fits = fits.open(archive+'/'+file_rad+'_S2D_SKYSUB_'+fiber+'.fits')
        else:
            e2ds_fits = fits.open(archive+'/'+file_rad+'_S2D_'+fiber+'.fits')
    else:
        if properties['use_ESO_telluric_correction']:
            e2ds_fits = fits.open(archive+'/'+file_rad+'_S2D_BLAZE_TELL_CORR_'+fiber+'.fits')
        else:
            e2ds_fits = fits.open(archive+'/'+file_rad+'_S2D_BLAZE_'+fiber+'.fits')

    input_dict['header']['e2ds'] = e2ds_fits[0].header

    input_dict['n_pixels'] = e2ds_fits[1].header['NAXIS1']
    input_dict['n_orders'] = len(selected_orders)

    input_dict['DPR_CATG'] = e2ds_fits[0].header[keywords['header_dpr_catg']]
    if e2ds_fits[0].header[keywords['header_dpr_catg']] != True:
        return

    if not skip_s1d:
        if properties['use_ESO_telluric_correction']:
            s1d_fits = fits.open(archive + '/' + file_rad + '_S1D_TELL_CORR_'+fiber+'.fits')
        elif properties['use_ESO_sky_correction']:
            s1d_fits = fits.open(archive + '/' + file_rad + '_S1D_SKYSUB_'+fiber+'.fits')
        else:
            s1d_fits = fits.open(archive + '/' + file_rad + '_S1D_'+fiber+'.fits')

        input_dict['header']['s1d'] = s1d_fits[0].header
        input_s1d['header']['s1d'] = s1d_fits[0].header

        temp_wave = s1d_fits[1].data['wavelength_air']
        temp_step = np.zeros_like(temp_wave)
        temp_step[1:] = np.diff(temp_wave)
        temp_step[0] = temp_step[1]
        sel_wave = (temp_wave >= 3799.9999) & (temp_wave <= 7850.0001)

        input_s1d['flux'] = s1d_fits[1].data['flux'][sel_wave]
        input_s1d['flux_err'] = s1d_fits[1].data['error'][sel_wave]
        input_s1d['wave'] = temp_wave[sel_wave]
        input_s1d['step'] = temp_step[sel_wave]
        input_s1d['size'] = np.size(input_s1d['wave'])

        s1d_fits.close()

    if not skip_ccf:
        if properties['use_ESO_telluric_correction']:
            ccf_fits = fits.open(archive+'/'+file_rad+'_CCF_TELL_CORR_'+fiber+'.fits')
        elif properties['use_ESO_sky_correction']:
            ccf_fits = fits.open(archive+'/'+file_rad+'_CCF_SKYSUB_'+fiber+'.fits')
        else:
            ccf_fits = fits.open(archive+'/'+file_rad+'_CCF_'+fiber+'.fits')
        input_dict['RVC'] = ccf_fits[0].header[keywords['header_rvc']]
        input_dict['header']['ccf'] = ccf_fits[0].header



    input_dict['BERV'] = e2ds_fits[0].header[keywords['header_berv']]
    input_dict['EXPTIME'] = e2ds_fits[0].header['EXPTIME']

    if properties['time_stamp'] == 'start_exposure':
        input_dict['BJD'] = e2ds_fits[0].header[keywords['header_bjd']] + input_dict['EXPTIME']/86400.
        input_dict['MJD'] = e2ds_fits[0].header[keywords['header_mjd']] + input_dict['EXPTIME']/86400.
    elif properties['time_stamp'] == 'mid_exposure':
        input_dict['BJD'] = e2ds_fits[0].header[keywords['header_bjd']]
        input_dict['MJD'] = e2ds_fits[0].header[keywords['header_mjd']]
    elif properties['time_stamp'] == 'end_exposure':
        input_dict['BJD'] = e2ds_fits[0].header[keywords['header_bjd']] - input_dict['EXPTIME']/86400.
        input_dict['MJD'] = e2ds_fits[0].header[keywords['header_mjd']] - input_dict['EXPTIME']/86400.
    else:
        print('*** please specify the relationship between epoch and exposure time - assuming mid-exposure epochs')
        input_dict['BJD'] = e2ds_fits[0].header[keywords['header_bjd']]
        input_dict['MJD'] = e2ds_fits[0].header[keywords['header_mjd']]

    input_dict['LST'] = e2ds_fits[0].header['LST']

    input_dict['AIRMASS'] = (e2ds_fits[0].header[keywords['airmass_alt_start']]
                                + e2ds_fits[0].header[keywords['airmass_alt_end']])/2.

    try:
        input_dict['AIRM_START'] = e2ds_fits[0].header[keywords['airmass_alt_start']]
        input_dict['AIRM_END'] = e2ds_fits[0].header[keywords['airmass_alt_end']]
    except:
        input_dict['AIRM_START'] = input_dict['AIRMASS'] -0.05
        input_dict['AIRM_END'] = max(1.00, input_dict['AIRMASS'] +0.05)

    input_dict['UTC'] = (input_dict['MJD'] - int(input_dict['MJD'])) * 86400.
    input_dict['HUMIDITY'] = e2ds_fits[0].header[keywords['humidity']]
    input_dict['PRESSURE'] = e2ds_fits[0].header[keywords['pressure']]
    input_dict['TEMPERATURE_EN'] = e2ds_fits[0].header[keywords['temperature_env']]
    input_dict['TEMPERATURE_M1'] = e2ds_fits[0].header[keywords['temperature_m1']]
    input_dict['ELEVATION'] = np.arcsin(1./input_dict['AIRMASS']) * (180./np.pi)

    input_dict['GEOELEV'] = properties['geoelev']
    input_dict['GEOLONG'] = properties['longitude']
    input_dict['GEOLAT'] = properties['latitude']

    input_dict['molecfit'] = properties['molecfit']

    try:
        try:
            input_dict['RA'] = e2ds_fits[0].header['RA-DEG']
            input_dict['DEC'] = e2ds_fits[0].header['DEC-DEG']
        except:
            input_dict['RA'] = e2ds_fits[0].header['RA-RAD'] * 180.00 / np.pi
            input_dict['DEC'] = e2ds_fits[0].header['DEC-RAD'] * 180.00 / np.pi  # weird choice of using DEC in hours
    except:
            input_dict['RA'] = e2ds_fits[0].header['RA']
            input_dict['DEC'] = e2ds_fits[0].header['DEC']

    try:
        input_dict['BERV'] = e2ds_fits[0].header[keywords['header_berv']]
        input_dict['EXPTIME'] = e2ds_fits[0].header['EXPTIME']

        if properties['time_stamp'] == 'start_exposure':
            input_dict['BJD'] = e2ds_fits[0].header[keywords['header_bjd']] + input_dict['EXPTIME']/86400.
            input_dict['MJD'] = e2ds_fits[0].header[keywords['header_mjd']] + input_dict['EXPTIME']/86400.
        elif properties['time_stamp'] == 'mid_exposure':
            input_dict['BJD'] = e2ds_fits[0].header[keywords['header_bjd']]
            input_dict['MJD'] = e2ds_fits[0].header[keywords['header_mjd']]
        elif properties['time_stamp'] == 'end_exposure':
            input_dict['BJD'] = e2ds_fits[0].header[keywords['header_bjd']] - input_dict['EXPTIME']/86400.
            input_dict['MJD'] = e2ds_fits[0].header[keywords['header_mjd']] - input_dict['EXPTIME']/86400.
        else:
            print('*** please specify the relationship between epoch and exposure time - assuming mid-exposure epochs')
            input_dict['BJD'] = e2ds_fits[0].header[keywords['header_bjd']]
            input_dict['MJD'] = e2ds_fits[0].header[keywords['header_mjd']]

        if properties['time_standard'] == 'UTC':
            input_dict['BJD']+= difference_utc2tdb(input_dict['MJD']+2400000.5)

        input_dict['LST'] = e2ds_fits[0].header['LST']


        input_dict['AIRMASS'] = (e2ds_fits[0].header[keywords['airmass_alt_start']]
                                    + e2ds_fits[0].header[keywords['airmass_alt_end']])/2.

        input_dict['UTC'] = e2ds_fits[0].header[keywords['header_utc']]
        #input_dict['UTC'] = (input_dict['MJD'] - int(input_dict['MJD'])) * 86400.
        input_dict['HUMIDITY'] = e2ds_fits[0].header[keywords['humidity']]
        input_dict['PRESSURE'] = e2ds_fits[0].header[keywords['pressure']]
        input_dict['TEMPERATURE_EN'] = e2ds_fits[0].header[keywords['temperature_env']]
        input_dict['TEMPERATURE_M1'] = e2ds_fits[0].header[keywords['temperature_m1']]
        input_dict['ELEVATION'] = np.arcsin(1./input_dict['AIRMASS']) * (180./np.pi)

        input_dict['GEOELEV'] = properties['geoelev']
        input_dict['GEOLONG'] = properties['longitude']
        input_dict['GEOLAT'] = properties['latitude']

        input_dict['molecfit'] = properties['molecfit']

        input_dict['RA'] = e2ds_fits[0].header['RA']
        input_dict['DEC'] = e2ds_fits[0].header['DEC']

        """""
        try:
            try:
                input_dict['RA'] = e2ds_fits[0].header['RA-DEG']
                input_dict['DEC'] = e2ds_fits[0].header['DEC-DEG']
            except:
                input_dict['RA'] = e2ds_fits[0].header['RA-RAD'] * 180.00 / np.pi
                input_dict['DEC'] = e2ds_fits[0].header['DEC-RAD'] * 180.00 / np.pi  # weird choice of using DEC in hours
        except:
                input_dict['RA'] = e2ds_fits[0].header['RA']
                input_dict['DEC'] = e2ds_fits[0].header['DEC']
        """""
    except:
        print('Keyword error in prepare_dataset - check the FITS header of your files')
        quit()
        pass

    # getting data

    input_dict['e2ds'] = e2ds_fits[1].data[properties['orders_regroup'], :][selected_orders, :]
    input_dict['e2ds_err'] = e2ds_fits[2].data[properties['orders_regroup'], :][selected_orders, :]

    if properties['apply_ESO_telluric_correction']:
        ssub_fits = fits.open(archive+'/'+file_rad+'_S2D_SKYSUB_'+fiber+'.fits')
        tell_fits = fits.open(archive+'/'+file_rad+'_S2D_TELL_CORR_'+fiber+'.fits')
        ssub_spec = ssub_fits[1].data[properties['orders_regroup'], :][selected_orders, :]
        tell_spec = tell_fits[1].data[properties['orders_regroup'], :][selected_orders, :]

        ssub_spec[ssub_spec == 0] = 1.0
        tell_spec[tell_spec == 0] = 1.0

        tell_spectrum_selected = tell_spec / ssub_spec
        ssub_fits.close()
        tell_fits.close()

        tell_spectrum_selected[tell_spectrum_selected == 0] = 1.0
        input_dict['e2ds'] /= tell_spectrum_selected
        input_dict['e2ds_err'] /= tell_spectrum_selected

    wave_with_berv =  e2ds_fits[5].data[properties['orders_regroup'], :][selected_orders, :]
    step_with_berv =  e2ds_fits[7].data[properties['orders_regroup'], :][selected_orders, :]

    #input_dict['wave'], input_dict['step'] = shift_wavelength_to_rest(wave_with_berv, step_with_berv, -input_dict['BERV'] )
    input_dict['wave'], input_dict['step'] = shift_wavelength(wave_with_berv, step_with_berv, -input_dict['BERV'] )
    input_dict['wave_size'] = e2ds_fits[1].header['NAXIS1']

    e2ds_fits.close()


    if not skip_ccf:
        ccf_fits.close()

    return input_dict,input_s1d



def ESPRESSO_get_instrument_keywords(night_dict):

    if night_dict['telescope'] == 'UT1':
        telescope = '1'
    elif night_dict['telescope'] == 'UT2':
        telescope = '2'
    elif night_dict['telescope'] == 'UT3':
        telescope = '3'
    elif night_dict['telescope'] == 'UT4':
        telescope = '4'
    else:
        telescope = ''

    keywords = {
        'header_rvc': 'HIERARCH ESO QC CCF RV',
        'header_berv': 'HIERARCH ESO QC BERV',
        'header_bjd': 'HIERARCH ESO QC BJD',
        'header_mjd': 'MJD-OBS', # MJD in days. This parameter is required for the retrieval of GDAS data
        'header_utc': 'UTC',

        'header_dpr_catg': 'HIERARCH ESO PRO SCIENCE',

        'airmass_alt_start': 'HIERARCH ESO TEL'+telescope+' AIRM START',
        'airmass_alt_end': 'HIERARCH ESO TEL'+telescope+' AIRM END',
        ## Telescope altitude is computed using the middle values obtained from airmass

        'humidity':'HIERARCH ESO TEL'+telescope+' AMBI RHUM', # Relative humidity in % for GEOELEV.
        #'pressure_start' : 'HIERARCH ESO TEL AMBI PRES START',
        #'pressure_end': 'HIERARCH ESO TEL AMBI PRES END',
        'pressure':'HIERARCH ESO TEL'+telescope+' AMBI PRES END',
        'temperature_env': 'HIERARCH ESO TEL'+telescope+' AMBI TEMP', #Ambient temperature in C for GEOELEV
        'temperature_m1': 'HIERARCH ESO TEL'+telescope+' TH M1 TEMP', # Temperature of primary mirror M1 in C (for emission spectra only)

    }

    properties = {
        # DRS-specific keywords
        'time_stamp': 'mid_exposure',
        'time_standard': 'BJD_TDB',

        # Observatory-specific keywords
        'geoelev': 2648.0, # meters
        'longitude' : -70.4051, # Tel geo longitude (+=East) (deg)
        'latitude' : -24.6276,  # Tel geo latitute (+=North) (deg)
        # Instrument-specific keyword
        'n_orders_A': 85,
        'n_orders_B': 85,
        'orders_BtoA':
            [0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
                10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
                20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
                30, 31, 32, 33, 34, 35, 36, 37, 38, 39,
                40, 41, 42, 43, 44, 45, 46, 47, 48, 49,
                50, 51, 52, 53, 54, 55, 56, 57, 58, 59,
                60, 61, 62, 63, 64, 65, 66, 67, 68, 69,
                70, 71, 72, 73, 74, 75, 76, 77, 78, 79,
                80, 81, 82, 83, 84],
            # after many experiments, I found out the easiest and more robust way to define
            # the order correspondence between fiber A anf B is just to write it down
        'red_ccd':
            [                               47, 48, 49,
                50, 51, 52, 53, 54, 55, 56, 57, 58, 59,
                60, 61, 62, 63, 64, 65, 66, 67, 68, 69,
                70, 71, 72, 73, 74, 75, 76, 77, 78, 79,
                80, 81, 82, 83, 84],
        'blue_ccd':
            [0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
             10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
             20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
             30, 31, 32, 33, 34, 35, 36, 37, 38, 39,
             40, 41, 42, 43, 44, 45, 46],
        'full_ccd':
            [0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
             10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
             20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
             30, 31, 32, 33, 34, 35, 36, 37, 38, 39,
             40, 41, 42, 43, 44, 45, 46, 47, 48, 49,
             50, 51, 52, 53, 54, 55, 56, 57, 58, 59,
             60, 61, 62, 63, 64, 65, 66, 67, 68, 69,
             70, 71, 72, 73, 74, 75, 76, 77, 78, 79,
             80, 81, 82, 83, 84],
        # The following are the input values used by Molecfit, taken from Allart+2017
        # for convenience, all the default values are listed here instead of being scattered into the code
        'molecfit': {
            'default_wstep': 0.01000, # default wavelength step size for the input stellar spectra
            'molecules': ['H2O', 'O2'],
            'ftol': 1e-9,
            'xtol': 1e-9,

            'cont_const': 1.0, # a0,  This value differs from Allart+2017 since we are using normalized spectra
            'cont_n': 3, # n_cont, Degree of coefficients for continuum fit
            'wlc_n': 2, # n_lambda, Polynomial degree of the refined wavelength solution
            'wlc_const': 0.0, # b0, Initial constant term for wavelength correction (shift relative to half  wavelength range)
            'res_gauss': 4.8, # omega_Gaussian,  Initial value for FWHM of Gaussian in pixels
            'kernfac': 15, #kernel_size, Size of Gaussian/Lorentzian/Voigtian kernel in FWHM

            'slitwidth': 1.00, # in arcseconds
            'pixelscale': 0.16,
        }
    }

    if night_dict['spectral_selection'] == 'odd':
        properties['orders_regroup'] = [
            1, 3, 5, 7, 9, 11, 13, 15, 17, 19,
            21, 23, 25, 27, 29, 31, 33, 35, 37, 39,
            41, 43, 45, 47, 49, 51, 53, 55, 57, 59,
            61, 63, 65, 67, 69, 71, 73, 75, 77, 79,
            81, 83, 85, 87, 89, 91, 93, 95, 97, 99,
            101, 103, 105, 107, 109, 111, 113, 115, 117, 119,
            121, 123, 125, 127, 129, 131, 133, 135, 137, 139,
            141, 143, 145, 147, 149, 151, 153, 155, 157, 159,
            161, 163, 165, 167, 169]

    if night_dict['spectral_selection'] == 'even':
        properties['orders_regroup'] = [
            0, 2, 4, 6, 8, 10, 12, 14, 16, 18,
            20, 22, 24, 26, 28, 30, 32, 34, 36, 38,
            40, 42, 44, 46, 48, 50, 52, 54, 56, 58,
            60, 62, 64, 66, 68, 70, 72, 74, 76, 78,
            80, 82, 84, 86, 88, 90, 92, 94, 96, 98,
            100, 102, 104, 106, 108, 110, 112, 114, 116, 118,
            120, 122, 124, 126, 128, 130, 132, 134, 136, 138,
            140, 142, 144, 146, 148, 150, 152, 154, 156, 158,
            160, 162, 164, 166, 168]

    properties['use_ESO_telluric_correction'] = night_dict.get('use_ESO_telluric_correction', True)
    properties['use_ESO_sky_correction'] = night_dict.get('use_ESO_sky_correction', True)
    properties['use_ESO_deblazed'] = night_dict.get('use_ESO_deblazed', True)
    properties['apply_ESO_telluric_correction'] = night_dict.get('apply_ESO_telluric_correction', False)

    if properties['apply_ESO_telluric_correction'] and properties['use_ESO_sky_correction']:
        print('*** WARNING: DAS telluric correction is performed on spectra with pre-applied sky correction.' \
              '             Please check the DAS documentation.')
        print('    You shoudl use the keyword use_ESO_telluric_correction to use directly ESO telluric correction')
        quit()

    if properties['use_ESO_telluric_correction'] and not properties['use_ESO_sky_correction']:
        print('*** WARNING: pre-applied telluric correction comes with pre-applied sky correction.' \
              '             a DAS file with only telluric correction is not available.' \
              '             Please check the DAS documentation.')
        print('    You can use the keyword apply_ESO_telluric_correction to apply the ESO telluric correction '\
              '    without using the pre-applied sky correction.')
        quit()

    if properties['use_ESO_sky_correction'] and not properties['use_ESO_deblazed']:
        print('*** WARNING: pre-applied sky correction is available only on deblazed files.' \
              '             As the computation of the relative efficiency between the two fibers is not yet implemented,' \
              '             at the moment is not possible to perform sky correction through the usual SLOPpy task.')

    return keywords, properties




from __future__ import print_function, division

from SLOPpy.instruments.common_DRSv3 import *

def HARPNv3_get_instrument_keywords():
    """ These definitions apply to DRS version 3.x """
    keywords = {
        'header_rvc': 'HIERARCH TNG DRS CCF RVC',
        'header_berv': 'HIERARCH TNG DRS BERV',
        'header_bjd': 'HIERARCH TNG DRS BJD',
        'header_mjd': 'MJD-OBS', # MJD in days. This parameter is required for the retrieval of GDAS data

        'header_blaze': 'HIERARCH TNG DRS BLAZE FILE',
        'header_ccd': 'HIERARCH TNG DRS CCD SIGDET',
        'header_conad': 'HIERARCH TNG DRS CCD CONAD',

        'header_dpr_catg': 'HIERARCH TNG DPR CATG',
        'header_dpr_type': 'HIERARCH TNG DPR TYPE',

        'header_deg_ll': 'HIERARCH TNG DRS CAL TH DEG LL',
        'header_coeff_ll': 'HIERARCH TNG DRS CAL TH COEFF LL',

        'airmass_alt_start': 'HIERARCH TNG TEL AIRM START',
        'airmass_alt_end': 'HIERARCH TNG TEL AIRM END',

        ## Telescope altitude is computed using the middle values obtained from airmass
        'humidity':'HIERARCH TNG METEO HUMIDITY', # Relative humidity in % for GEOELEV.
        'pressure':'HIERARCH TNG METEO PRESSURE',
        'temperature_env': 'HIERARCH TNG METEO TEMP10M', #Ambient temperature in C for GEOELEV
        'temperature_m1': 'HIERARCH TNG M1 CH1TEMP', # Temperature of primary mirror M1 in C (for emission spectra only)
    }

    properties = {
        # DRS-specific keywords
        'time_stamp': 'mid_exposure',
        'time_standard': 'UTC',

        # Observatory-specific keywords
        'geoelev': 2387.2, # meters
        'longitude' : -17.889, # Tel geo longitude (+=East) (deg)
        'latitude' : 28.754,  # Tel geo latitute (+=North) (deg)
        # Instrument-specific keyword
        'n_orders_A': 69,
        'n_orders_B': 69,
        'orders_BtoA':
            [0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
             10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
             20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
             30, 31, 32, 33, 34, 35, 36, 37, 38, 39,
             40, 41, 42, 43, 44, 45, 46, 47, 48, 49,
             50, 51, 52, 53, 54, 55, 56, 57, 58, 59,
             60, 61, 62, 63, 64, 65, 66, 67, 68],
            # after many experiments, I found out the easiest and more robust way to define
            # the order correspondence between fiber A anf B is just to write it down
        'red_ccd':
            [        42, 43, 44, 45, 46, 47, 48, 49,
             50, 51, 52, 53, 54, 55, 56, 57, 58, 59,
             60, 61, 62, 63, 64, 65, 66, 67, 68],
        'blue_ccd':
            [0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
             10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
             20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
             30, 31, 32, 33, 34, 35, 36, 37, 38, 39,
             40, 41],
        'full_ccd':
            [0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
             10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
             20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
             30, 31, 32, 33, 34, 35, 36, 37, 38, 39,
             40, 41, 42, 43, 44, 45, 46, 47, 48, 49,
             50, 51, 52, 53, 54, 55, 56, 57, 58, 59,
             60, 61, 62, 63, 64, 65, 66, 67, 68],
        # The following are the input values used by Molecfit, taken from Allart+2017
        # for convenience, all the default values are listed here instead of being scattered into the code
        'molecfit': {
            'default_wstep': 0.01000, # default wavelength step size for the input stellar spectra
            'molecules': ['H2O', 'O2'],
            'ftol': 1e-9,
            'xtol': 1e-9,

            'cont_const': 1.0, # a0,  This value differs from Allart+2017 since we are using normalized spectra
            'cont_n': 3, # n_cont, Degree of coefficients for continuum fit
            'wlc_n': 2, # n_lambda, Polynomial degree of the refined wavelength solution
            'wlc_const': 0.0, # b0, Initial constant term for wavelength correction (shift relative to half  wavelength range)
            'res_gauss': 4.8, # omega_Gaussian,  Initial value for FWHM of Gaussian in pixels
            'kernfac': 15, #kernel_size, Size of Gaussian/Lorentzian/Voigtian kernel in FWHM

            'slitwidth': 1.00, # in arcseconds
            'pixelscale': 0.16,
        }
    }

    return keywords, properties

# Shortcut from DRS-geenral to instrument-specific subroutine
def HARPN_DRSv3_get_calib_data(archive, file_rad, fiber='A', order_selection=None):

    keywords, properties = HARPNv3_get_instrument_keywords()
    return DRSv3_get_calib_data(archive, file_rad, keywords, properties, fiber=fiber, order_selection=order_selection)

# Shortcut from DRS-geenral to instrument-specific subroutine
def HARPN_DRSv3_get_input_data(archive, file_rad, mask, fiber='A', skip_ccf=None, skip_s1d=True, order_selection=None):

    keywords, properties = HARPNv3_get_instrument_keywords()
    return DRSv3_get_input_data(archive, file_rad, keywords, properties, mask, fiber=fiber, skip_ccf=skip_ccf, skip_s1d=skip_s1d, order_selection=order_selection)



import numpy as np
from SLOPpy.subroutines.math_functions import interpolate2d_grid_nocheck
from SLOPpy.subroutines.constants import *

def compute_single_line(wave_meshgrid, planet_RVsinusoid, planet_K, line_array):

    """ computing the spectral shift in RV """
    rv_shift = planet_K * planet_RVsinusoid

    line_model = np.ones(np.shape(wave_meshgrid), dtype= np.double)

    """ line_array is always structured in this way:
        line_array = np.empty(n_pams, n_lines)
        line_array[0, 0] = wavelength
        line_array[1, 0] = contrast
        line_array[2, 0] = FWHM
        line_array[3, 0] = winds
    """

    sigma =  line_array[2] / sigma2fwhm * line_array[0] / speed_of_light_km
    line_shifted = line_array[0] +  (rv_shift + line_array[3])  * line_array[0] / speed_of_light_km
    line_model -= line_array[1]  * np.exp(-(1./(2*sigma**2))*(wave_meshgrid-line_shifted)**2)

    return line_model


def compute_multiple_lines(wave_meshgrid, planet_RVsinusoid, planet_K, lines_array, n_lines):

    """ computing the spectral shift in RV """
    rv_shift = planet_K * planet_RVsinusoid

    lines_model = np.ones(np.shape(wave_meshgrid), dtype= np.double)

    for ii in range(0, n_lines):
        sigma =  lines_array[2,ii] / sigma2fwhm * lines_array[0,ii] / speed_of_light_km
        line_shifted = lines_array[0,ii] +  (rv_shift + lines_array[3,ii])  * lines_array[0,ii] / speed_of_light_km
        lines_model -= lines_array[1,ii]  * np.exp(-(1./(2*sigma**2))*(wave_meshgrid-line_shifted)**2)

    return lines_model


""" case  0: only one spectral line, default line parameters are contrast, FWHM, rv_shift """
def logprob_case00(theta,
            boundaries,
            wave_meshgrid,
            transmission_spec,
            transmission_spec_err,
            clv_rm_radius,
            clv_rm_grid,
            planet_RVsinusoid,
            lines_center,
            jitter_index,
            priors_dict,
            return_models=False
            ):

    """ check the boundaries """
    if ((theta < boundaries[:,0]) | (theta > boundaries[:,1])).any():
        return -np.inf

    """ unfolding jitter parameters """
    if jitter_index is None:
        i_j = -1
        jitter_array = 0.
        jitter_pams = 0.
    elif len(jitter_index) > 0:
        jitter_array = jitter_index * 0.
        n_jitter = int(np.amax(jitter_index) + 1)
        jitter_pams = np.empty(n_jitter)
        for i_j in range(0, n_jitter):
            sel = (jitter_index == i_j)
            jitter_array[sel] = theta[-1-i_j]
            jitter_pams[i_j] = theta[-1-i_j]
    else:
        i_j = 0
        jitter_array = wave_meshgrid*0. + theta[-1]
        jitter_pams = theta[-1]

    """ the second-last value after jitter is always the semi-amplitude of the planet
        the third-last value after jitter is always the planetary radius, if included in the model
    """
    planet_K = theta[-2-i_j]
    planet_R = theta[-3-i_j]

    """ computing interpolated model spectrum
    """
    clv_model = interpolate2d_grid_nocheck(planet_R, clv_rm_radius, clv_rm_grid)

    """ line_array is always structured in this way:
        line_array = np.empty(n_pams, n_lines)
        line_array[0, 0] = wavelength
        line_array[1, 0] = contrast
        line_array[2, 0] = FWHM
        line_array[3, 0] = winds
    """

    line_array = [lines_center[0], theta[0], theta[1], theta[2]]
    line_model = compute_single_line(wave_meshgrid, planet_RVsinusoid, planet_K, line_array)

    flux_res = transmission_spec / clv_model / line_model - 1.
    ferr_res = transmission_spec_err / clv_model / line_model

    if return_models:
        lines_array = np.empty([4, 1])
        lines_array[:, 0] = line_array
        return line_model, clv_model, lines_array, planet_K, planet_R, jitter_pams

    var_dict = {
        'planet_K': planet_K,
        'planet_R': planet_R,
        'FWHM': line_array[2],
        'winds': line_array[3],
    }
    log_prior = 0.
    for key_name, key_vals in priors_dict.items():
        log_prior +=  (-(var_dict[key_name] - key_vals[0]) ** 2 / (2 * key_vals[1] ** 2) - 0.5 * np.log(2*np.pi) - np.log(key_vals[1]))

    """ adding the jitter to error esitamets """
    env = 1.0 / (jitter_array ** 2.0 + ferr_res ** 2.0)
    return log_prior -0.5 * (np.size(flux_res) * np.log(2 * np.pi) +
                    np.sum((flux_res) ** 2 * env - np.log(env)))


""" case  1: only one spectral line, no winds """
def logprob_case01(theta,
            boundaries,
            wave_meshgrid,
            transmission_spec,
            transmission_spec_err,
            clv_rm_radius,
            clv_rm_grid,
            planet_RVsinusoid,
            lines_center,
            jitter_index,
            priors_dict,
            return_models=False
            ):

    """ check the boundaries """
    if ((theta < boundaries[:,0]) | (theta > boundaries[:,1])).any():
        return -np.inf

    """ unfolding jitter parameters """
    if jitter_index is None:
        i_j = -1
        jitter_array = 0.
        jitter_pams = 0.
    elif len(jitter_index) > 0:
        jitter_array = jitter_index * 0.
        n_jitter = int(np.amax(jitter_index) + 1)
        jitter_pams = np.empty(n_jitter)
        for i_j in range(0, n_jitter):
            sel = (jitter_index == i_j)
            jitter_array[sel] = theta[-1-i_j]
            jitter_pams[i_j] = theta[-1-i_j]
    else:
        i_j = 0
        jitter_array = wave_meshgrid*0. + theta[-1]
        jitter_pams = theta[-1]

    """ the second-last value after jitter is always the semi-amplitude of the planet
        the third-last value after jitter is always the planetary radius, if included in the model
    """
    planet_K = theta[-2-i_j]
    planet_R = theta[-3-i_j]

    """ computing interpolated model spectrum
    """
    clv_model = interpolate2d_grid_nocheck(planet_R, clv_rm_radius, clv_rm_grid)

    """ line_array is always structured in this way:
        line_array = np.empty(n_pams, n_lines)
        line_array[0, 0] = wavelength
        line_array[1, 0] = contrast
        line_array[2, 0] = FWHM
        line_array[3, 0] = winds
    """

    line_array = [lines_center[0], theta[0], theta[1], 0.000]
    line_model = compute_single_line(wave_meshgrid, planet_RVsinusoid, planet_K, line_array)

    flux_res = transmission_spec / clv_model / line_model - 1.
    ferr_res = transmission_spec_err / clv_model / line_model

    if return_models:
        lines_array = np.empty([4, 1])
        lines_array[:,0] = line_array
        return line_model, clv_model, lines_array, planet_K, planet_R, jitter_pams

    var_dict = {
        'planet_K': planet_K,
        'planet_R': planet_R,
        'FWHM': line_array[2],
        'winds': line_array[3],
    }
    log_prior = 0.
    for key_name, key_vals in priors_dict.items():
        log_prior +=  (-(var_dict[key_name] - key_vals[0]) ** 2 / (2 * key_vals[1] ** 2) - 0.5 * np.log(2*np.pi) - np.log(key_vals[1]))

    """ the last value is always the jitter """
    env = 1.0 / (jitter_array ** 2.0 + ferr_res ** 2.0)
    return log_prior -0.5 * (np.size(flux_res) * np.log(2 * np.pi) +
                    np.sum((flux_res) ** 2 * env - np.log(env)))


""" case  2: only one spectral line, no planetary radius dependance """
def logprob_case02(theta,
            boundaries,
            wave_meshgrid,
            transmission_spec,
            transmission_spec_err,
            clv_model,
            planet_RVsinusoid,
            lines_center,
            jitter_index,
            priors_dict,
            return_models=False
            ):

    """ check the boundaries """
    if ((theta < boundaries[:,0]) | (theta > boundaries[:,1])).any():
        return -np.inf

    """ unfolding jitter parameters """
    if jitter_index is None:
        i_j = -1
        jitter_array = 0.
        jitter_pams = 0.
    elif len(jitter_index) > 0:
        jitter_array = jitter_index * 0.
        n_jitter = int(np.amax(jitter_index) + 1)
        jitter_pams = np.empty(n_jitter)
        for i_j in range(0, n_jitter):
            sel = (jitter_index == i_j)
            jitter_array[sel] = theta[-1-i_j]
            jitter_pams[i_j] = theta[-1-i_j]
    else:
        i_j = 0
        jitter_array = wave_meshgrid*0. + theta[-1]
        jitter_pams = theta[-1]

    """ the second-last value after jitter is always the semi-amplitude of the planet
    """
    planet_K = theta[-2-i_j]

    line_array = [lines_center[0], theta[0], theta[1], theta[2]]
    line_model = compute_single_line(wave_meshgrid, planet_RVsinusoid, planet_K, line_array)

    flux_res = transmission_spec / clv_model / line_model - 1.
    ferr_res = transmission_spec_err / clv_model / line_model

    if return_models:
        lines_array = np.empty([4, 1])
        lines_array[:,0] = line_array
        return line_model, clv_model, lines_array, planet_K, 1.00000, jitter_pams

    var_dict = {
        'planet_K': planet_K,
        'planet_R': 1.000000,
        'FWHM': line_array[2],
        'winds': line_array[3],
    }
    log_prior = 0.
    for key_name, key_vals in priors_dict.items():
        log_prior +=  (-(var_dict[key_name] - key_vals[0]) ** 2 / (2 * key_vals[1] ** 2) - 0.5 * np.log(2*np.pi) - np.log(key_vals[1]))

    """ the last value is always the jitter """
    env = 1.0 / (jitter_array ** 2.0 + ferr_res ** 2.0)
    return log_prior -0.5 * (np.size(flux_res) * np.log(2 * np.pi) +
                    np.sum((flux_res) ** 2 * env - np.log(env)))


""" case  3: only one spectral line, no winds and no planetary radius dependance """
def logprob_case03(theta,
            boundaries,
            wave_meshgrid,
            transmission_spec,
            transmission_spec_err,
            clv_model,
            planet_RVsinusoid,
            lines_center,
            jitter_index,
            priors_dict,
            return_models=False
            ):

    """ check the boundaries """
    if ((theta < boundaries[:,0]) | (theta > boundaries[:,1])).any():
        return -np.inf

    """ unfolding jitter parameters """
    if jitter_index is None:
        i_j = -1
        jitter_array = 0.
        jitter_pams = 0.
    elif len(jitter_index) > 0:
        jitter_array = jitter_index * 0.
        n_jitter = int(np.amax(jitter_index) + 1)
        jitter_pams = np.empty(n_jitter)
        for i_j in range(0, n_jitter):
            sel = (jitter_index == i_j)
            jitter_array[sel] = theta[-1-i_j]
            jitter_pams[i_j] = theta[-1-i_j]
    else:
        i_j = 0
        jitter_array = wave_meshgrid*0. + theta[-1]
        jitter_pams = theta[-1]

    """ the second-last value after jitter is always the semi-amplitude of the planet
    """
    planet_K = theta[-2-i_j]

    line_array = [lines_center[0], theta[0], theta[1], 0.000]
    line_model = compute_single_line(wave_meshgrid, planet_RVsinusoid, planet_K, line_array)

    flux_res = transmission_spec / clv_model / line_model - 1.
    ferr_res = transmission_spec_err / clv_model / line_model

    if return_models:
        lines_array = np.empty([4, 1])
        lines_array[:,0] = line_array
        return line_model, clv_model, lines_array, planet_K, 1.00000, jitter_pams

    var_dict = {
        'planet_K': planet_K,
        'planet_R': 1.000000,
        'FWHM': line_array[2],
        'winds': line_array[3],
    }
    log_prior = 0.
    for key_name, key_vals in priors_dict.items():
        log_prior +=  (-(var_dict[key_name] - key_vals[0]) ** 2 / (2 * key_vals[1] ** 2) - 0.5 * np.log(2*np.pi) - np.log(key_vals[1]))

    """ the last value is always the jitter """
    env = 1.0 / (jitter_array ** 2.0 + ferr_res ** 2.0)
    return log_prior -0.5 * (np.size(flux_res) * np.log(2 * np.pi) +
                    np.sum((flux_res) ** 2 * env - np.log(env)))


""" case 10: more than one spectral lines, all line parameters are free and independent """
def logprob_case10(theta,
            boundaries,
            wave_meshgrid,
            transmission_spec,
            transmission_spec_err,
            clv_rm_radius,
            clv_rm_grid,
            planet_RVsinusoid,
            lines_center,
            jitter_index,
            priors_dict,
            return_models=False
            ):

    """ check the boundaries """
    if ((theta < boundaries[:,0]) | (theta > boundaries[:,1])).any():
        return -np.inf

    """ unfolding jitter parameters """
    if jitter_index is None:
        i_j = -1
        jitter_array = 0.
        jitter_pams = 0.
    elif len(jitter_index) > 0:
        jitter_array = jitter_index * 0.
        n_jitter = int(np.amax(jitter_index) + 1)
        jitter_pams = np.empty(n_jitter)
        for i_j in range(0, n_jitter):
            sel = (jitter_index == i_j)
            jitter_array[sel] = theta[-1-i_j]
            jitter_pams[i_j] = theta[-1-i_j]
    else:
        i_j = 0
        jitter_array = wave_meshgrid*0. + theta[-1]
        jitter_pams = theta[-1]

    """ the second-last value after jitter is always the semi-amplitude of the planet
        the third-last value after jitter is always the planetary radius, if included in the model
    """
    planet_K = theta[-2-i_j]
    planet_R = theta[-3-i_j]

    """ computing interpolated model spectrum
    """
    clv_model = interpolate2d_grid_nocheck(planet_R, clv_rm_radius, clv_rm_grid)

    """ line_array is always structured in this way:
        line_array = np.empty(n_pams, n_lines)
        line_array[0, 0] = wavelength
        line_array[1, 0] = contrast
        line_array[2, 0] = FWHM
        line_array[3, 0] = winds
    """

    n_lines = len(lines_center)
    lines_array = np.empty([4, n_lines])
    i_pams = 0
    for ii in range(0, n_lines):
        lines_array[0, ii] = lines_center[ii]
        lines_array[1, ii] = theta[i_pams]
        i_pams += 1
        lines_array[2, ii] = theta[i_pams]
        i_pams += 1
        lines_array[3, ii] = theta[i_pams]
        i_pams += 1

    lines_model = compute_multiple_lines(wave_meshgrid, planet_RVsinusoid, planet_K, lines_array, n_lines)

    flux_res = transmission_spec / clv_model / lines_model - 1.
    ferr_res = transmission_spec_err / clv_model / lines_model

    if return_models:
        return lines_model, clv_model, lines_array, planet_K, planet_R, jitter_pams

    var_dict = {
        'planet_K': planet_K,
        'planet_R': planet_R,
        'FWHM': lines_array[2,:],
        'winds': lines_array[3,:],
    }
    log_prior = 0.
    for key_name, key_vals in priors_dict.items():
        for var_value in np.atleast_1d(var_dict[key_name]):
            log_prior +=  (-(var_value - key_vals[0]) ** 2 / (2 * key_vals[1] ** 2) - 0.5 * np.log(2*np.pi) - np.log(key_vals[1]))

    """ the last value is always the jitter """
    env = 1.0 / (jitter_array ** 2.0 + ferr_res ** 2.0)
    return log_prior -0.5 * (np.size(flux_res) * np.log(2 * np.pi) +
                    np.sum((flux_res) ** 2 * env - np.log(env)))


""" case 11: more than one spectral lines, all lines are affected by the same wind """
def logprob_case11(theta,
            boundaries,
            wave_meshgrid,
            transmission_spec,
            transmission_spec_err,
            clv_rm_radius,
            clv_rm_grid,
            planet_RVsinusoid,
            lines_center,
            jitter_index,
            priors_dict,
            return_models=False
            ):

    """ check the boundaries """
    if ((theta < boundaries[:,0]) | (theta > boundaries[:,1])).any():
        return -np.inf

    """ unfolding jitter parameters """
    if jitter_index is None:
        i_j = -1
        jitter_array = 0.
        jitter_pams = 0.
    elif len(jitter_index) > 0:
        jitter_array = jitter_index * 0.
        n_jitter = int(np.amax(jitter_index) + 1)
        jitter_pams = np.empty(n_jitter)
        for i_j in range(0, n_jitter):
            sel = (jitter_index == i_j)
            jitter_array[sel] = theta[-1-i_j]
            jitter_pams[i_j] = theta[-1-i_j]
    else:
        i_j = 0
        jitter_array = wave_meshgrid*0. + theta[-1]
        jitter_pams = theta[-1]

    """ the second-last value after jitter is always the semi-amplitude of the planet
        the third-last value after jitter is always the planetary radius, if included in the model
    """
    planet_K = theta[-2-i_j]
    planet_R = theta[-3-i_j]

    """ computing interpolated model spectrum
    """
    clv_model = interpolate2d_grid_nocheck(planet_R, clv_rm_radius, clv_rm_grid)

    """ line_array is always structured in this way:
        line_array = np.empty(n_pams, n_lines)
        line_array[0, 0] = wavelength
        line_array[1, 0] = contrast
        line_array[2, 0] = FWHM
        line_array[3, 0] = winds
    """

    n_lines = len(lines_center)
    lines_array = np.empty([4, n_lines])
    i_pams = 0
    for ii in range(0, n_lines):
        lines_array[0, ii] = lines_center[ii]
        lines_array[1, ii] = theta[i_pams]
        i_pams += 1
        lines_array[2, ii] = theta[i_pams]
        i_pams += 1
        lines_array[3, ii] = theta[-4-i_j]

    lines_model = compute_multiple_lines(wave_meshgrid, planet_RVsinusoid, planet_K, lines_array, n_lines)

    flux_res = transmission_spec / clv_model / lines_model - 1.
    ferr_res = transmission_spec_err / clv_model / lines_model

    if return_models:
        return lines_model, clv_model, lines_array, planet_K, planet_R, jitter_pams

    var_dict = {
        'planet_K': planet_K,
        'planet_R': planet_R,
        'FWHM': lines_array[2,:],
        'winds': lines_array[3,:],
    }
    log_prior = 0.
    for key_name, key_vals in priors_dict.items():
        for var_value in np.atleast_1d(var_dict[key_name]):
            log_prior +=  (-(var_value - key_vals[0]) ** 2 / (2 * key_vals[1] ** 2) - 0.5 * np.log(2*np.pi) - np.log(key_vals[1]))

    """ the last value is always the jitter """
    env = 1.0 / (jitter_array ** 2.0 + ferr_res ** 2.0)
    return log_prior -0.5 * (np.size(flux_res) * np.log(2 * np.pi) +
                    np.sum((flux_res) ** 2 * env - np.log(env)))


""" case 12: more than one spectral lines, all lines have same FWHM """
def logprob_case12(theta,
            boundaries,
            wave_meshgrid,
            transmission_spec,
            transmission_spec_err,
            clv_rm_radius,
            clv_rm_grid,
            planet_RVsinusoid,
            lines_center,
            jitter_index,
            priors_dict,
            return_models=False
            ):

    """ check the boundaries """
    if ((theta < boundaries[:,0]) | (theta > boundaries[:,1])).any():
        return -np.inf

    """ unfolding jitter parameters """
    if jitter_index is None:
        i_j = -1
        jitter_array = 0.
        jitter_pams = 0.
    elif len(jitter_index) > 0:
        jitter_array = jitter_index * 0.
        n_jitter = int(np.amax(jitter_index) + 1)

        jitter_pams = np.empty(n_jitter)
        for i_j in range(0, n_jitter):
            sel = (jitter_index == i_j)

            jitter_array[sel] = theta[-1-i_j]
            jitter_pams[i_j] = theta[-1-i_j]
    else:
        i_j = 0
        jitter_array = wave_meshgrid*0. + theta[-1]
        jitter_pams = theta[-1]

    """ the second-last value after jitter is always the semi-amplitude of the planet
        the third-last value after jitter is always the planetary radius, if included in the model
    """
    planet_K = theta[-2-i_j]
    planet_R = theta[-3-i_j]

    """ computing interpolated model spectrum
    """
    clv_model = interpolate2d_grid_nocheck(planet_R, clv_rm_radius, clv_rm_grid)

    """ line_array is always structured in this way:
        line_array = np.empty(n_pams, n_lines)
        line_array[0, 0] = wavelength
        line_array[1, 0] = contrast
        line_array[2, 0] = FWHM
        line_array[3, 0] = winds
    """

    n_lines = len(lines_center)
    lines_array = np.empty([4, n_lines])
    i_pams = 0
    for ii in range(0, n_lines):
        lines_array[0, ii] = lines_center[ii]
        lines_array[1, ii] = theta[i_pams]
        i_pams += 1
        lines_array[2, ii] = theta[-4-i_j]
        lines_array[3, ii] = theta[i_pams]
        i_pams += 1

    lines_model = compute_multiple_lines(wave_meshgrid, planet_RVsinusoid, planet_K, lines_array, n_lines)

    flux_res = transmission_spec / clv_model / lines_model - 1.
    ferr_res = transmission_spec_err / clv_model / lines_model

    if return_models:
        return lines_model, clv_model, lines_array, planet_K, planet_R, jitter_pams

    var_dict = {
        'planet_K': planet_K,
        'planet_R': planet_R,
        'FWHM': lines_array[2,:],
        'winds': lines_array[3,:],
    }
    log_prior = 0.
    for key_name, key_vals in priors_dict.items():
        for var_value in np.atleast_1d(var_dict[key_name]):
            log_prior +=  (-(var_value - key_vals[0]) ** 2 / (2 * key_vals[1] ** 2) - 0.5 * np.log(2*np.pi) - np.log(key_vals[1]))

    """ the last value is always the jitter """
    env = 1.0 / (jitter_array ** 2.0 + ferr_res ** 2.0)
    return log_prior -0.5 * (np.size(flux_res) * np.log(2 * np.pi) +
                    np.sum((flux_res) ** 2 * env - np.log(env)))


""" case 13: more than one spectral lines, all lines are affected by the same wind and have same FWHM """
def logprob_case13(theta,
            boundaries,
            wave_meshgrid,
            transmission_spec,
            transmission_spec_err,
            clv_rm_radius,
            clv_rm_grid,
            planet_RVsinusoid,
            lines_center,
            jitter_index,
            priors_dict,
            return_models=False
            ):

    """ check the boundaries """
    if ((theta < boundaries[:,0]) | (theta > boundaries[:,1])).any():
        return -np.inf

    """ unfolding jitter parameters """
    if jitter_index is None:
        i_j = -1
        jitter_array = 0.
        jitter_pams = 0.
    elif len(jitter_index) > 0:
        jitter_array = jitter_index * 0.
        n_jitter = int(np.amax(jitter_index) + 1)
        jitter_pams = np.empty(n_jitter)
        for i_j in range(0, n_jitter):
            sel = (jitter_index == i_j)
            jitter_array[sel] = theta[-1-i_j]
            jitter_pams[i_j] = theta[-1-i_j]
    else:
        i_j = 0
        jitter_array = wave_meshgrid*0. + theta[-1]
        jitter_pams = theta[-1]

    """ the second-last value after jitter is always the semi-amplitude of the planet
        the third-last value after jitter is always the planetary radius, if included in the model
    """
    planet_K = theta[-2-i_j]
    planet_R = theta[-3-i_j]

    """ computing interpolated model spectrum
    """
    clv_model = interpolate2d_grid_nocheck(planet_R, clv_rm_radius, clv_rm_grid)

    """ line_array is always structured in this way:
        line_array = np.empty(n_pams, n_lines)
        line_array[0, 0] = wavelength
        line_array[1, 0] = contrast
        line_array[2, 0] = FWHM
        line_array[3, 0] = winds
    """

    n_lines = len(lines_center)
    lines_array = np.empty([4, n_lines])
    i_pams = 0
    for ii in range(0, n_lines):
        lines_array[0, ii] = lines_center[ii]
        lines_array[1, ii] = theta[i_pams]
        i_pams += 1
        lines_array[2, ii] = theta[-5-i_j]
        lines_array[3, ii] = theta[-4-i_j]

    lines_model = compute_multiple_lines(wave_meshgrid, planet_RVsinusoid, planet_K, lines_array, n_lines)

    flux_res = transmission_spec / clv_model / lines_model - 1.
    ferr_res = transmission_spec_err / clv_model / lines_model

    if return_models:
        return lines_model, clv_model, lines_array, planet_K, planet_R, jitter_pams

    var_dict = {
        'planet_K': planet_K,
        'planet_R': planet_R,
        'FWHM': lines_array[2,:],
        'winds': lines_array[3,:],
    }
    log_prior = 0.
    for key_name, key_vals in priors_dict.items():
        for var_value in np.atleast_1d(var_dict[key_name]):
            log_prior +=  (-(var_value - key_vals[0]) ** 2 / (2 * key_vals[1] ** 2) - 0.5 * np.log(2*np.pi) - np.log(key_vals[1]))

    """ the last value is always the jitter """
    env = 1.0 / (jitter_array ** 2.0 + ferr_res ** 2.0)
    return log_prior -0.5 * (np.size(flux_res) * np.log(2 * np.pi) +
                    np.sum((flux_res) ** 2 * env - np.log(env)))


""" case 14: more than one spectral lines, no winds """
def logprob_case14(theta,
            boundaries,
            wave_meshgrid,
            transmission_spec,
            transmission_spec_err,
            clv_rm_radius,
            clv_rm_grid,
            planet_RVsinusoid,
            lines_center,
            jitter_index,
            priors_dict,
            return_models=False
            ):

    """ check the boundaries """
    if ((theta < boundaries[:,0]) | (theta > boundaries[:,1])).any():
        return -np.inf

    """ unfolding jitter parameters """
    if jitter_index is None:
        i_j = -1
        jitter_array = 0.
        jitter_pams = 0.
    elif len(jitter_index) > 0:
        jitter_array = jitter_index * 0.
        n_jitter = int(np.amax(jitter_index) + 1)
        jitter_pams = np.empty(n_jitter)
        for i_j in range(0, n_jitter):
            sel = (jitter_index == i_j)
            jitter_array[sel] = theta[-1-i_j]
            jitter_pams[i_j] = theta[-1-i_j]
    else:
        i_j = 0
        jitter_array = wave_meshgrid*0. + theta[-1]
        jitter_pams = theta[-1]

    """ the second-last value after jitter is always the semi-amplitude of the planet
        the third-last value after jitter is always the planetary radius, if included in the model
    """
    planet_K = theta[-2-i_j]
    planet_R = theta[-3-i_j]

    """ computing interpolated model spectrum
    """
    clv_model = interpolate2d_grid_nocheck(planet_R, clv_rm_radius, clv_rm_grid)

    """ line_array is always structured in this way:
        line_array = np.empty(n_pams, n_lines)
        line_array[0, 0] = wavelength
        line_array[1, 0] = contrast
        line_array[2, 0] = FWHM
        line_array[3, 0] = winds
    """

    n_lines = len(lines_center)
    lines_array = np.empty([4, n_lines])
    i_pams = 0
    for ii in range(0, n_lines):
        lines_array[0, ii] = lines_center[ii]
        lines_array[1, ii] = theta[i_pams]
        i_pams += 1
        lines_array[2, ii] = theta[i_pams]
        i_pams += 1
        lines_array[3, ii] = 0.000

    lines_model = compute_multiple_lines(wave_meshgrid, planet_RVsinusoid, planet_K, lines_array, n_lines)

    flux_res = transmission_spec / clv_model / lines_model - 1.
    ferr_res = transmission_spec_err / clv_model / lines_model

    if return_models:
        return lines_model, clv_model, lines_array, planet_K, planet_R, jitter_pams

    var_dict = {
        'planet_K': planet_K,
        'planet_R': planet_R,
        'FWHM': lines_array[2,:],
        'winds': lines_array[3,:],
    }
    log_prior = 0.
    for key_name, key_vals in priors_dict.items():
        for var_value in np.atleast_1d(var_dict[key_name]):
            log_prior +=  (-(var_value - key_vals[0]) ** 2 / (2 * key_vals[1] ** 2) - 0.5 * np.log(2*np.pi) - np.log(key_vals[1]))

    """ the last value is always the jitter """
    env = 1.0 / (jitter_array ** 2.0 + ferr_res ** 2.0)
    return log_prior -0.5 * (np.size(flux_res) * np.log(2 * np.pi) +
                    np.sum((flux_res) ** 2 * env - np.log(env)))


""" case 15: more than one spectral lines, no winds, all lines have same FWHM """
def logprob_case15(theta,
            boundaries,
            wave_meshgrid,
            transmission_spec,
            transmission_spec_err,
            clv_rm_radius,
            clv_rm_grid,
            planet_RVsinusoid,
            lines_center,
            jitter_index,
            priors_dict,
            return_models=False
            ):

    """ check the boundaries """
    if ((theta < boundaries[:,0]) | (theta > boundaries[:,1])).any():
        return -np.inf

    """ unfolding jitter parameters """
    if jitter_index is None:
        i_j = -1
        jitter_array = 0.
        jitter_pams = 0.
    elif len(jitter_index) > 0:
        jitter_array = jitter_index * 0.
        n_jitter = int(np.amax(jitter_index) + 1)
        jitter_pams = np.empty(n_jitter)
        for i_j in range(0, n_jitter):
            sel = (jitter_index == i_j)
            jitter_array[sel] = theta[-1-i_j]
            jitter_pams[i_j] = theta[-1-i_j]
    else:
        i_j = 0
        jitter_array = wave_meshgrid*0. + theta[-1]
        jitter_pams = theta[-1]

    """ the second-last value after jitter is always the semi-amplitude of the planet
        the third-last value after jitter is always the planetary radius, if included in the model
    """
    planet_K = theta[-2-i_j]
    planet_R = theta[-3-i_j]

    """ computing interpolated model spectrum
    """
    clv_model = interpolate2d_grid_nocheck(planet_R, clv_rm_radius, clv_rm_grid)

    """ line_array is always structured in this way:
        line_array = np.empty(n_pams, n_lines)
        line_array[0, 0] = wavelength
        line_array[1, 0] = contrast
        line_array[2, 0] = FWHM
        line_array[3, 0] = winds
    """

    n_lines = len(lines_center)
    lines_array = np.empty([4, n_lines])
    i_pams = 0
    for ii in range(0, n_lines):
        lines_array[0, ii] = lines_center[ii]
        lines_array[1, ii] = theta[i_pams]
        i_pams += 1
        lines_array[2, ii] = theta[-4-i_j]
        lines_array[3, ii] = 0.000

    lines_model = compute_multiple_lines(wave_meshgrid, planet_RVsinusoid, planet_K, lines_array, n_lines)

    flux_res = transmission_spec / clv_model / lines_model - 1.
    ferr_res = transmission_spec_err / clv_model / lines_model

    if return_models:
        return lines_model, clv_model, lines_array, planet_K, planet_R, jitter_pams

    var_dict = {
        'planet_K': planet_K,
        'planet_R': planet_R,
        'FWHM': lines_array[2,:],
        'winds': lines_array[3,:],
    }
    log_prior = 0.
    for key_name, key_vals in priors_dict.items():
        for var_value in np.atleast_1d(var_dict[key_name]):
            log_prior +=  (-(var_value - key_vals[0]) ** 2 / (2 * key_vals[1] ** 2) - 0.5 * np.log(2*np.pi) - np.log(key_vals[1]))

    """ the last value is always the jitter """
    env = 1.0 / (jitter_array ** 2.0 + ferr_res ** 2.0)
    return log_prior -0.5 * (np.size(flux_res) * np.log(2 * np.pi) +
                    np.sum((flux_res) ** 2 * env - np.log(env)))






""" case 20: more than one spectral lines, no Rp dependance, all line parameters are free and independent """
def logprob_case20(theta,
            boundaries,
            wave_meshgrid,
            transmission_spec,
            transmission_spec_err,
            clv_model,
            planet_RVsinusoid,
            lines_center,
            jitter_index,
            priors_dict,
            return_models=False
            ):

    """ check the boundaries """
    if ((theta < boundaries[:,0]) | (theta > boundaries[:,1])).any():
        return -np.inf

    """ unfolding jitter parameters """
    if jitter_index is None:
        i_j = -1
        jitter_array = 0.
        jitter_pams = 0.
    elif len(jitter_index) > 0:
        jitter_array = jitter_index * 0.
        n_jitter = int(np.amax(jitter_index) + 1)
        jitter_pams = np.empty(n_jitter)
        for i_j in range(0, n_jitter):
            sel = (jitter_index == i_j)
            jitter_array[sel] = theta[-1-i_j]
            jitter_pams[i_j] = theta[-1-i_j]
    else:
        i_j = 0
        jitter_array = wave_meshgrid*0. + theta[-1]
        jitter_pams = theta[-1]

    """ the second-last value after jitter is always the semi-amplitude of the planet
        the third-last value after jitter is always the planetary radius, if included in the model
    """
    planet_K = theta[-2-i_j]

    """ line_array is always structured in this way:
        line_array = np.empty(n_pams, n_lines)
        line_array[0, 0] = wavelength
        line_array[1, 0] = contrast
        line_array[2, 0] = FWHM
        line_array[3, 0] = winds
    """

    n_lines = len(lines_center)
    lines_array = np.empty([4, n_lines])
    i_pams = 0
    for ii in range(0, n_lines):
        lines_array[0, ii] = lines_center[ii]
        lines_array[1, ii] = theta[i_pams]
        i_pams += 1
        lines_array[2, ii] = theta[i_pams]
        i_pams += 1
        lines_array[3, ii] = theta[i_pams]
        i_pams += 1

    lines_model = compute_multiple_lines(wave_meshgrid, planet_RVsinusoid, planet_K, lines_array, n_lines)

    flux_res = transmission_spec / clv_model / lines_model - 1.
    ferr_res = transmission_spec_err / clv_model / lines_model

    if return_models:
        return lines_model, clv_model, lines_array, planet_K, 1.0000, jitter_pams

    var_dict = {
        'planet_K': planet_K,
        'planet_R': 1.00000,
        'FWHM': lines_array[2,:],
        'winds': lines_array[3,:],
    }
    log_prior = 0.
    for key_name, key_vals in priors_dict.items():
        for var_value in np.atleast_1d(var_dict[key_name]):
            log_prior +=  (-(var_value - key_vals[0]) ** 2 / (2 * key_vals[1] ** 2) - 0.5 * np.log(2*np.pi) - np.log(key_vals[1]))

    """ the last value is always the jitter """
    env = 1.0 / (jitter_array ** 2.0 + ferr_res ** 2.0)
    return log_prior -0.5 * (np.size(flux_res) * np.log(2 * np.pi) +
                    np.sum((flux_res) ** 2 * env - np.log(env)))


""" case 21: more than one spectral lines, no Rp dependance, all lines are affected by the same wind """
def logprob_case21(theta,
            boundaries,
            wave_meshgrid,
            transmission_spec,
            transmission_spec_err,
            clv_model,
            planet_RVsinusoid,
            lines_center,
            jitter_index,
            priors_dict,
            return_models=False
            ):

    """ check the boundaries """
    if ((theta < boundaries[:,0]) | (theta > boundaries[:,1])).any():
        return -np.inf

    """ unfolding jitter parameters """
    if jitter_index is None:
        i_j = -1
        jitter_array = 0.
        jitter_pams = 0.
    elif len(jitter_index) > 0:
        jitter_array = jitter_index * 0.
        n_jitter = int(np.amax(jitter_index) + 1)
        jitter_pams = np.empty(n_jitter)
        for i_j in range(0, n_jitter):
            sel = (jitter_index == i_j)
            jitter_array[sel] = theta[-1-i_j]
            jitter_pams[i_j] = theta[-1-i_j]
    else:
        i_j = 0
        jitter_array = wave_meshgrid*0. + theta[-1]
        jitter_pams = theta[-1]

    """ the second-last value after jitter is always the semi-amplitude of the planet
        the third-last value after jitter is always the planetary radius, if included in the model
    """
    planet_K = theta[-2-i_j]

    """ line_array is always structured in this way:
        line_array = np.empty(n_pams, n_lines)
        line_array[0, 0] = wavelength
        line_array[1, 0] = contrast
        line_array[2, 0] = FWHM
        line_array[3, 0] = winds
    """

    n_lines = len(lines_center)
    lines_array = np.empty([4, n_lines])
    i_pams = 0
    for ii in range(0, n_lines):
        lines_array[0, ii] = lines_center[ii]
        lines_array[1, ii] = theta[i_pams]
        i_pams += 1
        lines_array[2, ii] = theta[i_pams]
        i_pams += 1
        lines_array[3, ii] = theta[-3-i_j]

    lines_model = compute_multiple_lines(wave_meshgrid, planet_RVsinusoid, planet_K, lines_array, n_lines)

    flux_res = transmission_spec / clv_model / lines_model - 1.
    ferr_res = transmission_spec_err / clv_model / lines_model

    if return_models:
        return lines_model, clv_model, lines_array, planet_K, 1.0000, jitter_pams

    var_dict = {
        'planet_K': planet_K,
        'planet_R': 1.000000,
        'FWHM': lines_array[2,:],
        'winds': lines_array[3,:],
    }
    log_prior = 0.
    for key_name, key_vals in priors_dict.items():
        for var_value in np.atleast_1d(var_dict[key_name]):
            log_prior +=  (-(var_value - key_vals[0]) ** 2 / (2 * key_vals[1] ** 2) - 0.5 * np.log(2*np.pi) - np.log(key_vals[1]))

    """ the last value is always the jitter """
    env = 1.0 / (jitter_array ** 2.0 + ferr_res ** 2.0)
    return log_prior -0.5 * (np.size(flux_res) * np.log(2 * np.pi) +
                    np.sum((flux_res) ** 2 * env - np.log(env)))


""" case 22: more than one spectral lines, no Rp dependance, all lines have same FWHM """
def logprob_case22(theta,
            boundaries,
            wave_meshgrid,
            transmission_spec,
            transmission_spec_err,
            clv_model,
            planet_RVsinusoid,
            lines_center,
            jitter_index,
            priors_dict,
            return_models=False
            ):

    """ check the boundaries """
    if ((theta < boundaries[:,0]) | (theta > boundaries[:,1])).any():
        return -np.inf

    """ unfolding jitter parameters """
    if jitter_index is None:
        i_j = -1
        jitter_array = 0.
        jitter_pams = 0.
    elif len(jitter_index) > 0:
        jitter_array = jitter_index * 0.
        n_jitter = int(np.amax(jitter_index) + 1)
        jitter_pams = np.empty(n_jitter)
        for i_j in range(0, n_jitter):
            sel = (jitter_index == i_j)
            jitter_array[sel] = theta[-1-i_j]
            jitter_pams[i_j] = theta[-1-i_j]
    else:
        i_j = 0
        jitter_array = wave_meshgrid*0. + theta[-1]
        jitter_pams = theta[-1]

    """ the second-last value after jitter is always the semi-amplitude of the planet
        the third-last value after jitter is always the planetary radius, if included in the model
    """
    planet_K = theta[-2-i_j]

    """ line_array is always structured in this way:
        line_array = np.empty(n_pams, n_lines)
        line_array[0, 0] = wavelength
        line_array[1, 0] = contrast
        line_array[2, 0] = FWHM
        line_array[3, 0] = winds
    """

    n_lines = len(lines_center)
    lines_array = np.empty([4, n_lines])
    i_pams = 0
    for ii in range(0, n_lines):
        lines_array[0, ii] = lines_center[ii]
        lines_array[1, ii] = theta[i_pams]
        i_pams += 1
        lines_array[2, ii] = theta[-3-i_j]
        lines_array[3, ii] = theta[i_pams]
        i_pams += 1

    lines_model = compute_multiple_lines(wave_meshgrid, planet_RVsinusoid, planet_K, lines_array, n_lines)

    flux_res = transmission_spec / clv_model / lines_model - 1.
    ferr_res = transmission_spec_err / clv_model / lines_model

    if return_models:
        return lines_model, clv_model, lines_array, planet_K, 1.0000, jitter_pams

    var_dict = {
        'planet_K': planet_K,
        'planet_R': 1.000000,
        'FWHM': lines_array[2,:],
        'winds': lines_array[3,:],
    }
    log_prior = 0.
    for key_name, key_vals in priors_dict.items():
        for var_value in np.atleast_1d(var_dict[key_name]):
            log_prior +=  (-(var_value - key_vals[0]) ** 2 / (2 * key_vals[1] ** 2) - 0.5 * np.log(2*np.pi) - np.log(key_vals[1]))

    """ the last value is always the jitter """
    env = 1.0 / (jitter_array ** 2.0 + ferr_res ** 2.0)
    return log_prior -0.5 * (np.size(flux_res) * np.log(2 * np.pi) +
                    np.sum((flux_res) ** 2 * env - np.log(env)))


""" case 23: more than one spectral lines, no Rp dependance, all lines are affected by the same wind and have same FWHM """
def logprob_case23(theta,
            boundaries,
            wave_meshgrid,
            transmission_spec,
            transmission_spec_err,
            clv_model,
            planet_RVsinusoid,
            lines_center,
            jitter_index,
            priors_dict,
            return_models=False
            ):

    """ check the boundaries """
    if ((theta < boundaries[:,0]) | (theta > boundaries[:,1])).any():
        return -np.inf

    """ unfolding jitter parameters """
    if jitter_index is None:
        i_j = -1
        jitter_array = 0.
        jitter_pams = 0.
    elif len(jitter_index) > 0:
        jitter_array = jitter_index * 0.
        n_jitter = int(np.amax(jitter_index) + 1)
        jitter_pams = np.empty(n_jitter)
        for i_j in range(0, n_jitter):
            sel = (jitter_index == i_j)
            jitter_array[sel] = theta[-1-i_j]
            jitter_pams[i_j] = theta[-1-i_j]
    else:
        i_j = 0
        jitter_array = wave_meshgrid*0. + theta[-1]
        jitter_pams = theta[-1]

    """ the second-last value after jitter is always the semi-amplitude of the planet
        the third-last value after jitter is always the planetary radius, if included in the model
    """
    planet_K = theta[-2-i_j]

    """ line_array is always structured in this way:
        line_array = np.empty(n_pams, n_lines)
        line_array[0, 0] = wavelength
        line_array[1, 0] = contrast
        line_array[2, 0] = FWHM
        line_array[3, 0] = winds
    """

    n_lines = len(lines_center)
    lines_array = np.empty([4, n_lines])
    i_pams = 0

    for ii in range(0, n_lines):
        lines_array[0, ii] = lines_center[ii]
        lines_array[1, ii] = theta[i_pams]
        i_pams += 1
        lines_array[2, ii] = theta[-4-i_j]
        lines_array[3, ii] = theta[-3-i_j]

    lines_model = compute_multiple_lines(wave_meshgrid, planet_RVsinusoid, planet_K, lines_array, n_lines)

    flux_res = transmission_spec / clv_model / lines_model - 1.
    ferr_res = transmission_spec_err / clv_model / lines_model

    if return_models:
        return lines_model, clv_model, lines_array, planet_K, 1.0000, jitter_pams

    var_dict = {
        'planet_K': planet_K,
        'planet_R': 1.000000,
        'FWHM': lines_array[2,:],
        'winds': lines_array[3,:],
    }
    log_prior = 0.
    for key_name, key_vals in priors_dict.items():
        for var_value in np.atleast_1d(var_dict[key_name]):
            log_prior +=  (-(var_value - key_vals[0]) ** 2 / (2 * key_vals[1] ** 2) - 0.5 * np.log(2*np.pi) - np.log(key_vals[1]))

    """ the last value is always the jitter """
    env = 1.0 / (jitter_array ** 2.0 + ferr_res ** 2.0)
    return log_prior -0.5 * (np.size(flux_res) * np.log(2 * np.pi) +
                    np.sum((flux_res) ** 2 * env - np.log(env)))


""" case 24: more than one spectral lines, no Rp dependance, no winds """
def logprob_case24(theta,
            boundaries,
            wave_meshgrid,
            transmission_spec,
            transmission_spec_err,
            clv_model,
            planet_RVsinusoid,
            lines_center,
            jitter_index,
            priors_dict,
            return_models=False
            ):

    """ check the boundaries """
    if ((theta < boundaries[:,0]) | (theta > boundaries[:,1])).any():
        return -np.inf

    """ unfolding jitter parameters """
    if jitter_index is None:
        i_j = -1
        jitter_array = 0.
        jitter_pams = 0.
    elif len(jitter_index) > 0:
        jitter_array = jitter_index * 0.
        n_jitter = int(np.amax(jitter_index) + 1)
        jitter_pams = np.empty(n_jitter)
        for i_j in range(0, n_jitter):
            sel = (jitter_index == i_j)
            jitter_array[sel] = theta[-1-i_j]
            jitter_pams[i_j] = theta[-1-i_j]
    else:
        i_j = 0
        jitter_array = wave_meshgrid*0. + theta[-1]
        jitter_pams = theta[-1]

    """ the second-last value after jitter is always the semi-amplitude of the planet
        the third-last value after jitter is always the planetary radius, if included in the model
    """
    planet_K = theta[-2-i_j]

    """ line_array is always structured in this way:
        line_array = np.empty(n_pams, n_lines)
        line_array[0, 0] = wavelength
        line_array[1, 0] = contrast
        line_array[2, 0] = FWHM
        line_array[3, 0] = winds
    """

    n_lines = len(lines_center)
    lines_array = np.empty([4, n_lines])
    i_pams = 0
    for ii in range(0, n_lines):
        lines_array[0, ii] = lines_center[ii]
        lines_array[1, ii] = theta[i_pams]
        i_pams += 1
        lines_array[2, ii] = theta[i_pams]
        i_pams += 1
        lines_array[3, ii] = 0.000

    lines_model = compute_multiple_lines(wave_meshgrid, planet_RVsinusoid, planet_K, lines_array, n_lines)

    flux_res = transmission_spec / clv_model / lines_model - 1.
    ferr_res = transmission_spec_err / clv_model / lines_model

    if return_models:
        return lines_model, clv_model, lines_array, planet_K, 1.0000, jitter_pams

    var_dict = {
        'planet_K': planet_K,
        'planet_R': 1.000000,
        'FWHM': lines_array[2,:],
        'winds': lines_array[3,:],
    }
    log_prior = 0.
    for key_name, key_vals in priors_dict.items():
        for var_value in np.atleast_1d(var_dict[key_name]):
            log_prior +=  (-(var_value - key_vals[0]) ** 2 / (2 * key_vals[1] ** 2) - 0.5 * np.log(2*np.pi) - np.log(key_vals[1]))

    """ the last value is always the jitter """
    env = 1.0 / (jitter_array ** 2.0 + ferr_res ** 2.0)
    return log_prior -0.5 * (np.size(flux_res) * np.log(2 * np.pi) +
                    np.sum((flux_res) ** 2 * env - np.log(env)))


""" case 25: more than one spectral lines, no Rp dependance, no winds, all lines have same FWHM """
def logprob_case25(theta,
            boundaries,
            wave_meshgrid,
            transmission_spec,
            transmission_spec_err,
            clv_model,
            planet_RVsinusoid,
            lines_center,
            jitter_index,
            priors_dict,
            return_models=False
            ):

    """ check the boundaries """
    if ((theta < boundaries[:,0]) | (theta > boundaries[:,1])).any():
        return -np.inf

    """ unfolding jitter parameters """
    if jitter_index is None:
        i_j = -1
        jitter_array = 0.
        jitter_pams = 0.
    elif len(jitter_index) > 0:
        jitter_array = jitter_index * 0.
        n_jitter = int(np.amax(jitter_index) + 1)
        jitter_pams = np.empty(n_jitter)
        for i_j in range(0, n_jitter):
            sel = (jitter_index == i_j)
            jitter_array[sel] = theta[-1-i_j]
            jitter_pams[i_j] = theta[-1-i_j]
    else:
        i_j = 0
        jitter_array = wave_meshgrid*0. + theta[-1]
        jitter_pams = theta[-1]

    """ the second-last value after jitter is always the semi-amplitude of the planet
        the third-last value after jitter is always the planetary radius, if included in the model
    """
    planet_K = theta[-2-i_j]

    """ line_array is always structured in this way:
        line_array = np.empty(n_pams, n_lines)
        line_array[0, 0] = wavelength
        line_array[1, 0] = contrast
        line_array[2, 0] = FWHM
        line_array[3, 0] = winds
    """

    n_lines = len(lines_center)
    lines_array = np.empty([4, n_lines])
    i_pams = 0
    for ii in range(0, n_lines):
        lines_array[0, ii] = lines_center[ii]
        lines_array[1, ii] = theta[i_pams]
        i_pams += 1
        lines_array[2, ii] = theta[-3-i_j]
        lines_array[3, ii] = 0.000

    lines_model = compute_multiple_lines(wave_meshgrid, planet_RVsinusoid, planet_K, lines_array, n_lines)

    flux_res = transmission_spec / clv_model / lines_model - 1.
    ferr_res = transmission_spec_err / clv_model / lines_model

    if return_models:
        return lines_model, clv_model, lines_array, planet_K, 1.0000, jitter_pams

    var_dict = {
        'planet_K': planet_K,
        'planet_R': 1.000000,
        'FWHM': lines_array[2,:],
        'winds': lines_array[3,:],
    }
    log_prior = 0.
    for key_name, key_vals in priors_dict.items():
        for var_value in np.atleast_1d(var_dict[key_name]):
            log_prior +=  (-(var_value - key_vals[0]) ** 2 / (2 * key_vals[1] ** 2) - 0.5 * np.log(2*np.pi) - np.log(key_vals[1]))

    """ the last value is always the jitter """
    env = 1.0 / (jitter_array ** 2.0 + ferr_res ** 2.0)
    return log_prior -0.5 * (np.size(flux_res) * np.log(2 * np.pi) +
                    np.sum((flux_res) ** 2 * env - np.log(env)))



from __future__ import print_function, division
from SLOPpy.subroutines.common import *
from SLOPpy.subroutines.spectral_subroutines import *
from SLOPpy.subroutines.io_subroutines import *
from SLOPpy.subroutines.fit_subroutines import *
from SLOPpy.subroutines.shortcuts import *
from SLOPpy.subroutines.rebin_subroutines import *
from SLOPpy.subroutines.clv_rm_subroutines import *
from astropy.convolution import convolve, Box1DKernel

__all__ = ['compute_spectra_lightcurve',
           'compute_spectra_lightcurve_clv_rm_correction',
           'plot_spectra_lightcurve',
           'plot_spectra_lightcurve_clv_rm_correction']


def compute_spectra_lightcurve_clv_rm_correction(config_in, lines_label):
    compute_spectra_lightcurve(config_in, lines_label)


def plot_spectra_lightcurve_clv_rm_correction(config_in, night_input=''):
    plot_spectra_lightcurve(config_in, night_input)


def compute_spectra_lightcurve(config_in, lines_label):


    subroutine_name = 'spectra_lightcurve'

    sampler_name = 'emcee'

    do_average_instead_of_sum = True

    night_dict = from_config_get_nights(config_in)
    #instrument_dict = from_config_get_instrument(config_in)
    #system_dict = from_config_get_system(config_in)
    planet_dict = from_config_get_planet(config_in)

    spectral_lines = from_config_get_spectral_lines(config_in)
    lines_dict = spectral_lines[lines_label] 
    clv_rm_correction = lines_dict.get('clv_rm_correction', True)


    # from_config_get_transmission_lightcurve(config_in)
    #lightcurve_dict = from_config_get_transmission_lightcurve(config_in)

    shared_data = load_from_cpickle('shared', config_in['output'])

    """ Using the MCMC fit range to define the transmission spectrum region """

    shared_selection = (shared_data['coadd']['wave'] >= lines_dict['range'][0]) \
        & (shared_data['coadd']['wave'] < lines_dict['range'][1])

    processed_template = {
        'subroutine': subroutine_name,
        'range': lines_dict['range'],
        'wave': shared_data['coadd']['wave'][shared_selection],
        'step': shared_data['coadd']['step'][shared_selection],
        'size': int(np.sum(shared_selection)),
    }


    # doublet sodium in the lab reference frame

    """
    C stands for central
    """
    C_bands = {}
    for passband_key, passband_val in spectral_lines['passbands'].items():
        C_bands[passband_key] = {}
        for line_key, line_val in spectral_lines['lines'].items():
            C_bands[passband_key][line_key] = (np.abs(shared_data['coadd']['wave'] - line_val) < passband_val / 2.)

    """
    S stands for side
    """
    S_bands = {}
    for band_key, band_val in spectral_lines['continuum'].items():
        S_bands[band_key] = (shared_data['coadd']['wave'] >= band_val[0]) & (shared_data['coadd']['wave'] <= band_val[1])

    """
        The transit phase [0-1] is divided in N (=5) bins. Two arrays are computed:
        - transit_in_bins: array with the boundaries of the bins, size=N+1
        - transit_in_step: average size of the bin, size=1
    """
    transit_in_bins = np.linspace(
        -planet_dict['transit_duration'][0]/2./planet_dict['period'][0],
        planet_dict['transit_duration'][0]/2./planet_dict['period'][0],
        6
    )
    transit_in_step = np.average(transit_in_bins[1:]-transit_in_bins[:-1])

    for night in night_dict:

        try:
            lightcurve = load_from_cpickle(subroutine_name, config_in['output'], night)
            print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Retrieved'))
            continue
        except:
            print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Computing'))
            print()


        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)

        calib_data = load_from_cpickle('calibration_fibA', config_in['output'], night)
        input_data = retrieve_observations( config_in['output'], night, lists['observations'])
        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)

        processed = processed_template.copy()

        lightcurve = {
            'subroutine': subroutine_name,
            'arrays': {
                'observations': {
                    'obs_name': np.zeros(len(lists['observations']), dtype=str),
                    'phase': np.zeros(len(lists['observations'])),
                },
                'transit_in': {},
                'transit_out': {},
            },
            'C_bands': C_bands,
            'S_bands': S_bands,
            'average': {},
            'bins': {
                'transit_in_bins': transit_in_bins,
                'transit_in_step': transit_in_step
            }
        }
        """ Adding the C-bands arrays to the dictionary"""

        for band_key in C_bands:
            lightcurve['arrays']['observations']['ratio_' + band_key] = np.zeros([len(lists['observations']), 2])

        transit_out_flag = np.zeros(len(lists['observations']), dtype=bool)
        transit_in_flag = np.zeros(len(lists['observations']), dtype=bool)

        if clv_rm_correction:
            try:
                clv_rm_models = load_from_cpickle('clv_rm_models', config_in['output'], night, lines_label)
            except (FileNotFoundError, IOError):
                clv_rm_models = load_from_cpickle('clv_rm_models', config_in['output'], night)


        for n_obs, obs in enumerate( lists['observations']):

            processed[obs] = {}
            lightcurve[obs] = {}

            processed[obs]['rescaling'], \
            processed[obs]['rescaled'], \
            processed[obs]['rescaled_err'] = perform_rescaling(
                input_data[obs]['wave'], input_data[obs]['e2ds'], input_data[obs]['e2ds_err'],
                observational_pams['wavelength_rescaling'])

            preserve_flux = input_data[obs].get('absolute_flux', True)

            processed[obs]['uncorrected'] = \
                rebin_2d_to_1d(input_data[obs]['wave'],
                               input_data[obs]['step'],
                               input_data[obs]['e2ds'],
                               calib_data['blaze'],
                               processed['wave'],
                               processed['step'],
                               preserve_flux=preserve_flux,
                               rv_shift=observational_pams[obs]['rv_shift_ORF2SRF'])

            processed[obs]['uncorrected_err'] = \
                rebin_2d_to_1d(input_data[obs]['wave'],
                               input_data[obs]['step'],
                               input_data[obs]['e2ds_err'],
                               calib_data['blaze'],
                               processed['wave'],
                               processed['step'],
                               preserve_flux=preserve_flux,
                               rv_shift=observational_pams[obs]['rv_shift_ORF2SRF'])

            if clv_rm_correction:

                rv_shift = 0.0 # we always stay in SRF
                correction, _ = clv_rm_correction_factor_computation(
                    clv_rm_modelling, shared_data['coadd']['wave'], shared_data['coadd']['step'], rv_shift, obs)

                processed[obs]['clv_rm_correction'] = correction
                processed[obs]['corrected'] /= correction
                processed[obs]['corrected_err'] /= correction





            try:
                phase_internal = (observational_pams[obs]['BJD'] - night_dict[night]['time_of_transit'][0])/planet_dict['period'][0]
            except:
                phase_internal = (observational_pams[obs]['BJD'] - night_dict[night]['time_of_transit'])/planet_dict['period'][0]


            processed[obs]['bands'] = {
                'phase': phase_internal
            }

            s_integrated = 0.000
            s_sigmaq_sum = 0.00
            n_bands = 0.00
            for band_key, band_val in S_bands.items():
                if do_average_instead_of_sum:
                    processed[obs]['bands'][band_key] =  \
                        [np.average(processed[obs]['rebinned'][band_val]),
                         np.sum((processed[obs]['rebinned_err'][band_val])**2)
                         / len(processed[obs]['rebinned_err'][band_val])**2]
                else:
                    processed[obs]['bands'][band_key] =  \
                        [np.sum(processed[obs]['rebinned'][band_val]),
                         np.sum((processed[obs]['rebinned_err'][band_val])**2)]

                s_integrated += processed[obs]['bands'][band_key][0]
                s_sigmaq_sum += processed[obs]['bands'][band_key][1]
                n_bands += 1

            s_integrated *= (2. / n_bands)
            s_sigmaq_sum *= (2. / n_bands)**2

            s_factor_term = np.power(s_integrated, -2.0)

            for band_key, band_dict in C_bands.items():
                processed[obs]['bands'][band_key] = {}

                c_integrated = 0.000
                c_sigmaq_sum = 0.000
                n_bands = 0.00

                for line_key, line_val in band_dict.items():
                    if do_average_instead_of_sum:
                        processed[obs]['bands'][band_key][line_key] = \
                            [np.average(processed[obs]['rebinned'][line_val]),
                             np.sum((processed[obs]['rebinned_err'][line_val]) ** 2)
                             / len(processed[obs]['rebinned_err'][line_val]) ** 2]
                    else:
                        processed[obs]['bands'][band_key][line_key] = \
                            [np.sum(processed[obs]['rebinned'][line_val]),
                             np.sum((processed[obs]['rebinned_err'][line_val]) ** 2)]

                    c_integrated += processed[obs]['bands'][band_key][line_key][0]
                    c_sigmaq_sum += processed[obs]['bands'][band_key][line_key][1]
                    n_bands += 1

                c_integrated *= (2. / n_bands)
                c_sigmaq_sum *= (2. / n_bands) ** 2

                ratio = c_integrated / s_integrated
                lightcurve[obs]['ratio_' + band_key] = [ratio, ratio * np.sqrt( c_sigmaq_sum / c_integrated ** 2 +
                                                                                s_sigmaq_sum / s_integrated ** 2)]
                                                        #np.sqrt(s_factor_term
                                                        #      * (c_sigmaq_sum + ratio**2 * s_sigmaq_sum))]

                lightcurve['arrays']['observations']['ratio_' + band_key][n_obs, :] = \
                    lightcurve[obs]['ratio_' + band_key][:]

            lightcurve[obs]['phase'] = processed[obs]['bands']['phase']
            lightcurve['arrays']['observations']['obs_name'][n_obs] = obs
            lightcurve['arrays']['observations']['phase'][n_obs] = lightcurve[obs]['phase']

            if obs in lists['transit_out']:
                transit_out_flag[n_obs] = True
            else:
                transit_in_flag[n_obs] = True


        for band_key in C_bands:
            lightcurve['arrays']['rescaling_' + band_key] = \
                np.average(lightcurve['arrays']['observations']['ratio_' + band_key][transit_out_flag, 0], axis=0)

        sorting_index = np.argsort(lightcurve['arrays']['observations']['phase'])

        transit_out_flag = transit_out_flag[sorting_index]
        transit_in_flag = transit_in_flag[sorting_index]

        lightcurve['arrays']['observations']['obs_name'] = lightcurve['arrays']['observations']['obs_name'][sorting_index]
        lightcurve['arrays']['observations']['phase'] = lightcurve['arrays']['observations']['phase'][sorting_index]

        lightcurve['arrays']['transit_in']['obs_name'] = lightcurve['arrays']['observations']['obs_name'][transit_in_flag]
        lightcurve['arrays']['transit_in']['phase'] = lightcurve['arrays']['observations']['phase'][transit_in_flag]

        lightcurve['arrays']['transit_out']['obs_name'] = lightcurve['arrays']['observations']['obs_name'][transit_out_flag]
        lightcurve['arrays']['transit_out']['phase'] = lightcurve['arrays']['observations']['phase'][transit_out_flag]

        for band_key in C_bands:
            lightcurve['arrays']['observations']['ratio_' + band_key] = \
                lightcurve['arrays']['observations']['ratio_' + band_key][sorting_index] \
                 / lightcurve['arrays']['rescaling_' + band_key]
            lightcurve['arrays']['transit_in']['ratio_' + band_key] = \
                lightcurve['arrays']['observations']['ratio_' + band_key][transit_in_flag]
            lightcurve['arrays']['transit_out']['ratio_' + band_key] = \
                lightcurve['arrays']['observations']['ratio_' + band_key][transit_out_flag]

            avg_out, avg_out_sq = \
                np.average(lightcurve['arrays']['transit_out']['ratio_' + band_key][:, 0],
                           weights=1./(lightcurve['arrays']['transit_out']['ratio_' + band_key][:, 1])**2,
                           returned=True)
            avg_in, avg_in_sq = \
                np.average(lightcurve['arrays']['transit_in']['ratio_' + band_key][:, 0],
                           weights=1. / (lightcurve['arrays']['transit_in']['ratio_' + band_key][:, 1]) ** 2,
                           returned=True)

            lightcurve['average'][band_key] = {
                'average_out': np.asarray([avg_out, 1./np.power(avg_out_sq, 0.5)]),
                'average_in': np.asarray([avg_in, 1. / np.power(avg_in_sq, 0.5)]),
            }

            delta_fac = \
                    lightcurve['average'][band_key]['average_in'][0]/lightcurve['average'][band_key]['average_out'][0]
            delta_err = delta_fac * np.sqrt(
                (lightcurve['average'][band_key]['average_out'][1]
                 / lightcurve['average'][band_key]['average_out'][0]) ** 2
                + (lightcurve['average'][band_key]['average_in'][1]
                   / lightcurve['average'][band_key]['average_in'][0]) ** 2)

            lightcurve['average'][band_key]['delta'] = np.asarray([(1.-delta_fac)*100., delta_err*100.])

        lightcurve['arrays']['observations']['transit_out_flag'] = transit_out_flag
        lightcurve['arrays']['observations']['transit_in_flag'] = transit_in_flag


        """ Compute the duration of the pre-transit observations, using as scale
            the number of bins, with the same size as those used inside the transit.
            
            The value is given by the difference of the phase of the beginning of the transit minus
            the phase of the first observation, keeping in mind that the centre of the transit has phase = 0
            
            An additional bin is added if there are observations left out from the actual number of bins
        """
        pre_duration = transit_in_bins[0] - lightcurve['arrays']['transit_out']['phase'][0]
        if pre_duration > 0:
            nsteps_pre = int(pre_duration/transit_in_step)
            if pre_duration % transit_in_step > 0.0:
                nsteps_pre += 1
        else:
            nsteps_pre = 0

        """ same as pre-transit, but suing the post-transit instead"""
        post_duration = lightcurve['arrays']['transit_out']['phase'][-1] - transit_in_bins[-1]
        if post_duration > 0:
            nsteps_post = int(post_duration / transit_in_step)
            if post_duration % transit_in_step > 0.0:
                nsteps_post += 1
        else:
            nsteps_post = 0

        """ THe full array with both in-transit and out-transit phase, built in such a way that the 
            - the lower boundary of the first in-transit bin corresponds to the beginning of the transit
            - the upper boundary of the last in-transit bin corresponds to the end of the transit
        """
        transit_bins = np.arange(transit_in_bins[0]-nsteps_pre*transit_in_step,
                                 transit_in_bins[-1] + (nsteps_post+1.1) * transit_in_step,
                                 transit_in_step)

        lightcurve['binned'] = {
            'observations': {
                'phase': np.zeros(len(transit_bins)),
            },
            'transit_in': {},
            'transit_out': {},
        }
        for band_key in C_bands:
            lightcurve['binned']['observations']['ratio_' + band_key] = np.zeros([len(transit_bins), 2])

        transit_out_flag = np.zeros(len(transit_bins), dtype=bool)
        transit_in_flag = np.zeros(len(transit_bins), dtype=bool)

        n_a = 0
        for nb in range(0, len(transit_bins)-1):
            sel = (lightcurve['arrays']['observations']['phase'] >= transit_bins[nb]) \
                   & (lightcurve['arrays']['observations']['phase'] < transit_bins[nb+1])

            if np.sum(sel) <= 0: continue
            lightcurve['binned']['observations']['phase'][n_a] = np.average(lightcurve['arrays']['observations']['phase'][sel])

            for band_key in C_bands:

                lightcurve['binned']['observations']['ratio_' + band_key][n_a, 0], sum_weights = np.average(
                    lightcurve['arrays']['observations']['ratio_' + band_key][sel, 0],
                    weights=1. / lightcurve['arrays']['observations']['ratio_' + band_key][sel, 1]**2,
                    returned=True)

                lightcurve['binned']['observations']['ratio_' + band_key][n_a, 1] = np.sqrt(1. / sum_weights)

            if np.abs(lightcurve['binned']['observations']['phase'][n_a]) >= \
                planet_dict['transit_duration'][0]/2./planet_dict['period'][0]:
                transit_out_flag[n_a] = True
            else:
                transit_in_flag[n_a] = True

            n_a += 1 # bins actually computed

        lightcurve['binned']['transit_in']['phase'] = lightcurve['binned']['observations']['phase'][transit_in_flag]
        lightcurve['binned']['transit_out']['phase'] = lightcurve['binned']['observations']['phase'][transit_out_flag]
        lightcurve['binned']['observations']['phase'] = lightcurve['binned']['observations']['phase'][:n_a]

        for band_key in C_bands:
            lightcurve['binned']['transit_in']['ratio_' + band_key] = \
                lightcurve['binned']['observations']['ratio_' + band_key][transit_in_flag, :]
            lightcurve['binned']['transit_out']['ratio_' + band_key] = \
                lightcurve['binned']['observations']['ratio_' + band_key][transit_out_flag, :]
            lightcurve['binned']['observations']['ratio_' + band_key] = \
                lightcurve['binned']['observations']['ratio_' + band_key][:n_a, :]

        save_to_cpickle(subroutine_name+'_processed', processed, config_in['output'], night)
        save_to_cpickle(subroutine_name, lightcurve, config_in['output'], night)


def plot_spectra_lightcurve(config_in, night_input='', clv_rm_correction=False):
    import matplotlib.pyplot as plt

    if clv_rm_correction:
        subroutine_name = 'spectra_lightcurve_clv_rm_correction'
    else:
        subroutine_name = 'spectra_lightcurve'

    night_dict = from_config_get_nights(config_in)

    if night_input=='':
        night_list = night_dict
    else:
        night_list = np.atleast_1d(night_input)

    for night in night_list:

        """ Retrieving the analysis"""
        try:
            lightcurve = load_from_cpickle(subroutine_name, config_in['output'], night)
            print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Plotting'))
        except:
            print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Skipped'))
            continue

        #observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)
        C_bands = lightcurve['C_bands']


        print()
        for band_key in C_bands:
            print("Night: {0:s}   Band: {1:s}   Delta:{2:8.4f} +- {3:8.4f} [%]".format(night, band_key,
                  lightcurve['average'][band_key]['delta'][0], lightcurve['average'][band_key]['delta'][1]))

        for band_key in C_bands:


            plt.figure(figsize=(12, 6))
            plt.title('Spectra lightcurve - night {0:s} \n {1:s}'.format(night, band_key))
            plt.errorbar(lightcurve['arrays']['observations']['phase'],
                        lightcurve['arrays']['observations']['ratio_' + band_key][:,0]*100 -100.,
                         yerr= lightcurve['arrays']['observations']['ratio_' + band_key][:,1]*100 ,
                         fmt='.', c='k', alpha=0.25, label='observations')
            plt.errorbar(lightcurve['binned']['observations']['phase'],
                        lightcurve['binned']['observations']['ratio_' + band_key][:, 0]*100 -100.,
                         yerr= lightcurve['binned']['observations']['ratio_' + band_key][:,1]*100 ,
                         fmt='.', c='k', alpha=1.0, label='observations')

            plt.axvspan(-1, lightcurve['bins']['transit_in_bins'][0], alpha=0.25, color='green')
            plt.axvspan(lightcurve['bins']['transit_in_bins'][-1], 1., alpha=0.25, color='green')

            plt.axhline(0, c='C1')
            plt.xlim(lightcurve['arrays']['observations']['phase'][0]-0.01,
                     lightcurve['arrays']['observations']['phase'][-1]+0.01)
            plt.xlabel('orbital phase')
            plt.ylabel('$\mathcal{R}$ - 1. [%]')
            plt.legend()
            plt.show()

    print()
